<?php

/**  
 *
 *       dilbo - digital logbook for Rowing and Canoeing
 *       -----------------------------------------------
 *       https://www.dilbo.org
 *
 * Copyright  2023-2024  Martin Glade
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */
class Dilbo_config
{

    /**
     * Additional set of configuration files for the tree mode, See tfyh_config.php for mandatory ones
     * ($mandatory_cfg_files).
     */
    public static $extra_config_files = ["clubSettings" => ".appObjects.Club",
            "uiSettings" => ".appObjects.UI","efa2objects" => null,"efa2tables" => null,
            "efaCloudTables" => null,"efa2import" => null,"efaCloudImport" => null
    ];

    /**
     * Editable configuration files, used by jspost to check authorisation.
     */
    public static $editable_config_files = ["appSettings","clubSettings","uiSettings"
    ];

    /**
     * Mapping of all efaCloud configuration parameters into the dilbo configuration tree
     */
    public static $efaCloud_to_dilbo_config = [
            "ClubworkAgeDays" => ".appSettings.archiving.clubwork_age_days",
            "DamageAgeDays" => ".appSettings.archiving.damage_age_days",
            "MessageAgeDays" => ".appSettings.archiving.message_age_days",
            "PersonsAgeDays" => ".appSettings.archiving.persons_age_days",
            "ReservationAgeDays" => ".appSettings.archiving.reservation_age_days",
            "TripAgeDays" => ".appSettings.archiving.trip_age_days",
            "mail_footer" => ".appSettings.communication.mailer.mail_footer",
            "mail_subscript" => ".appSettings.communication.mailer.mail_subscript",
            "system_mail_sender" => ".appSettings.communication.mailer.system_mail_sender",
            "pdf_footer_text" => ".appSettings.communication.pdf.footer_text",
            "pers_logbook_cols" => ".appSettings.communication.persLogbook.pers_logbook_cols",
            "pers_logbook_table" => ".appSettings.communication.persLogbook.pers_logbook_table",
            "pers_logbook_td" => ".appSettings.communication.persLogbook.pers_logbook_td",
            "pers_logbook_th" => ".appSettings.communication.persLogbook.pers_logbook_th",
            "pers_logbook_tr" => ".appSettings.communication.persLogbook.pers_logbook_tr",
            "public_notavailable" => ".appSettings.communication.publicInfo.notavailable",
            "public_notusable" => ".appSettings.communication.publicInfo.notusable",
            "public_onthewater" => ".appSettings.communication.publicInfo.onthewater",
            "public_tripdata_BoatAffix" => ".appSettings.communication.publicInfo.publicTripdata.boat_affix",
            "public_tripdata_BoatName" => ".appSettings.communication.publicInfo.publicTripdata.boat_name",
            "public_tripdata_BoatType" => ".appSettings.communication.publicInfo.publicTripdata.boat_type",
            "public_tripdata_CrewGroups" => ".appSettings.communication.publicInfo.publicTripdata.crew_groups",
            "public_tripdata_Destination" => ".appSettings.communication.publicInfo.publicTripdata.destination",
            "public_tripdata_Distance" => ".appSettings.communication.publicInfo.publicTripdata.distance",
            "public_tripdata_EntryId" => ".appSettings.communication.publicInfo.publicTripdata.entry_id",
            "public_tripdata_StartTime" => ".appSettings.communication.publicInfo.publicTripdata.start_time",
            "public_reserved" => ".appSettings.communication.publicInfo.reserved",
            "synch_check_period" => ".appSettings.communication.synchronization.synch_check_period",
            "synch_period" => ".appSettings.communication.synchronization.synch_period",
            "configured_jobs" => ".appSettings.Maintenance.configured_jobs",
            "PurgeDeletedAgeDays" => ".appSettings.Maintenance.purge_deleted_age_days",
            "AVVdatum" => ".appSettings.operations.date_of_cdp_agreement",
            "debug_support" => ".appSettings.operations.debug_on","Hoster" => ".appSettings.operations.hoster",
            "Betriebsverantwortlich" => ".appSettings.operations.operations_responsible",
            "acronym" => ".club.acronym","language_code" => ".club.habits.language",
            "efa_NameFormat" => ".club.habits.name_format",
            "sports_year_start" => ".club.habits.sports_year_start","app_url" => false,"backup" => false,
            "current_logbook" => false,"current_logbook2" => false,"current_logbook3" => false,
            "current_logbook4" => false,"mail_schriftwart" => false,"notify_admin_message_to" => false,
            "notify_damage_to" => false,"notify_damage_unusable_only" => false,
            "notify_reservation_to" => false,"pdf_document_author" => false,"reference_client" => false,
            "Verein" => false
    ];

    /**
     * the handling options for efa/efaCloud to dilbo value mapping.
     */
    private static $efa_efaCloud_to_dilbo_handlings = ["add_time","copy","copy_as_update","copy_bookname",
            "copy_distance","copy_list","create_boat_variants","id_at_other_table","map_damage_number",
            "merge_crew","millis_to_microtime","skip","use_name_if_empty","use_name_list_if_empty"
    ];

    /**
     * the efa/efaCloud to dilbo table mapping. Records may be merged, as for efa2boats + efa2boatstatus =>
     * assets, and efa2persons + efaCloudUsers => persons. The policy is a ;-separated String with
     * "target;handling;if_empty;add", i.e. the target dilbo field, the applicable handling, a value to be
     * used if empty and a value to be added (datetime join). Handling is one of
     * $efa_efaClud_to_dilbo_tables_handling.
     */
    private static $efa_efaCloud_to_dilbo_mapping = [
            "efa2boatdamages" => ["." => "damages","Damage" => "number;map_damage_number;;",
                    "BoatId" => "asset_uuid;copy;;","Description" => "description;copy;;",
                    "Severity" => "severity;copy;;","ReportedByPersonId" => "reported_by;copy;;",
                    "ReportDate" => "reported_on;add_time;;ReportTime","FixedByPersonId" => "fixed_by;copy;;",
                    "FixDate" => "fixed_on;add_time;;FixTime","RepairCosts" => "repair_costs;copy;;",
                    "LogbookText" => "logbook_text;copy;;","Fixed" => "fixed;copy;;","Notes" => "notes;copy;;",
                    "Claim" => "claim;copy;;","LastModified" => "modified;millis_to_microtime;;"
            ],
            "efa2boatreservations" => ["." => "reservations","Reservation" => "number;copy;;",
                    "BoatId" => "asset_uuid;copy;;","Reason" => "reason;copy;;","Type" => "type;copy;;",
                    "DateFrom" => "start;add_time;;TimeFrom","DateTo" => "end;add_time;DateFrom;TimeTo",
                    "DayOfWeek" => "day_of_week;copy;;","PersonId" => "person_uuid;copy;;",
                    "Contact" => "contact;copy;;","LastModified" => "modified;millis_to_microtime;;"
            ],
            "efa2boats" => ["." => "assets","Id" => "uuid;copy;;","Name" => "name;copy;;",
                    "NameAffix" => "name_affix;copy;;","Owner" => "asset_owner;copy;;",
                    "LastVariant" => "last_variant;copy;;","DefaultVariant" => "default_variant;copy;;",
                    "TypeVariant" => "variant_indices;create_boat_variants;;",
                    "AllowedGroupIdList" => "allowed_group_uuids;copy;;",
                    "RequiredGroupId" => "required_group_uuid;copy;;",
                    "MaxNotInGroup" => "max_not_in_group;copy;;","Manufacturer" => "manufacturer;copy;;",
                    "Model" => "model;copy;;","MaxCrewWeight" => "max_crew_weight;copy;;",
                    "ManufactionDate" => "manufaction_date;copy;;","SerialNo" => "serial_no;copy;;",
                    "PurchaseDate" => "purchase_date;copy;;","PurchasePrice" => "purchase_price;copy;;",
                    "PurchasePriceCurrency" => "purchase_price_currency;copy;;",
                    "SellingDate" => "selling_date;copy;;","SellingPrice" => "selling_price;copy;;",
                    "InsuranceValue" => "insurance_value;copy;;","DefaultCrewId" => "default_crew_uuid;copy;;",
                    "DefaultSessionType" => "default_session_type;copy;;",
                    "DefaultDestinationId" => "default_destination_uuid;copy;;",
                    "OnlyWithBoatCaptain" => "only_with_boat_captain;copy;;",
                    "ExcludeFromStatistics" => "exclude_from_statistics;copy;;","EfbId" => "efb_id;copy;;",
                    "FreeUse1" => "free_use_1;copy;;","FreeUse2" => "free_use_2;copy;;",
                    "FreeUse3" => "free_use_3;copy;;","ValidFrom" => "valid_from;millis_to_microtime;;",
                    "InvalidFrom" => "invalid_from;millis_to_microtime;;","Invisible" => "invisible;copy;;",
                    "Deleted" => "deleted;copy;;","LastModified" => "modified;millis_to_microtime;;"
            ],
            "efa2boatstatus" => ["." => "assets","BoatId" => "uuid;id_at_other_table;;",
                    "BaseStatus" => "base_status;copy_as_update;;"
            ],
            "efa2clubwork" => ["." => "workbook","Clubworkbookname" => "workbookname;copy_bookname;;",
                    "Id" => "uuid;copy;;","PersonId" => "person_uuid;copy;;","Date" => "date;copy;;",
                    "Description" => "description;copy;;","Hours" => "hours;copy;;","Flag" => "flag;copy;;",
                    "Approved" => "approved;copy;;","LastModified" => "modified;millis_to_microtime;;"
            ],
            "efa2crews" => ["." => "crews","Id" => "uuid;copy;;","Name" => "name;copy;;",
                    "CoxId" => "cox_uuid;copy;;","BoatCaptain" => "boat_captain;copy;;",
                    "Crew1Id" => "member_uuids;merge_crew;;",
                    "LastModified" => "modified;millis_to_microtime;;"
            ],
            "efa2destinations" => ["." => "destinations","Id" => "uuid;copy;;","Name" => "name;copy;;",
                    "Start" => "start_location;copy;;","End" => "end_location;copy;;",
                    "StartIsBoathouse" => "start_is_boathouse;copy;;","Roundtrip" => "roundtrip;copy;;",
                    "DestinationAreas" => "destination_areas;copy;;","PassedLocks" => "passed_locks;copy;;",
                    "Distance" => "distance;copy_distance;;",
                    "OnlyInBoathouseId" => "only_in_boathouse_id;copy;;",
                    "WatersIdList" => "waters_uuids;copy;;","ValidFrom" => "valid_from;millis_to_microtime;;",
                    "InvalidFrom" => "invalid_from;millis_to_microtime;;","Invisible" => "invisible;copy;;",
                    "Deleted" => "deleted;copy;;","LastModified" => "modified;millis_to_microtime;;"
            ],
            "efa2fahrtenabzeichen" => ["." => "badges","PersonId" => "personid;copy;;",
                    "Abzeichen" => "abzeichen;copy;;","AbzeichenAB" => "abzeichenab;copy;;",
                    "Kilometer" => "kilometer;copy;;","KilometerAB" => "kilometerab;copy;;",
                    "Fahrtenheft" => "fahrtenheft;copy;;","GUI_VORNAME" => "gui_vorname;copy;;",
                    "GUI_NACHNAME" => "gui_nachname;copy;;","GUI_JAHRGANG" => "gui_jahrgang;copy;;",
                    "GUI_ANZABZEICHEN" => "gui_anzabzeichen;copy;;","GUI_ANZKM" => "gui_anzkm;copy;;",
                    "GUI_ANZABZEICHENAB" => "gui_anzabzeichenab;copy;;","GUI_ANZKMAB" => "gui_anzkmab;copy;;",
                    "GUI_LETZTESJAHR" => "gui_letztesjahr;copy;;","GUI_LETZTEKM" => "gui_letztekm;copy;;",
                    "GUI_LETZTESDATUM" => "gui_letztesdatum;copy;;","GUI_VERSION" => "gui_version;copy;;",
                    "GUI_SCHLUESSEL" => "gui_schluessel;copy;;","GUI_SIGNATUR" => "gui_signatur;copy;;",
                    "GUI_STATUS" => "gui_status;copy;;","LastModified" => "modified;millis_to_microtime;;"
            ],
            "efa2groups" => ["." => "groups","Id" => "uuid;copy;;","Name" => "name;copy;;",
                    "Color" => "color;copy;;","MemberIdList" => "member_uuids;copy;;",
                    "ValidFrom" => "valid_from;millis_to_microtime;;",
                    "InvalidFrom" => "invalid_from;millis_to_microtime;;","Invisible" => "invisible;copy;;",
                    "Deleted" => "deleted;copy;;","LastModified" => "modified;millis_to_microtime;;"
            ],
            "efa2logbook" => ["." => "logbook","Logbookname" => "logbookname;copy_bookname;;",
                    "EntryId" => "number;copy;;","Date" => "start;add_time;;StartTime",
                    "EndDate" => "end;add_time;Date;EndTime",
                    "BoatId" => "asset_uuid;use_name_if_empty;BoatName;",
                    "BoatVariant" => "asset_variant;copy;;","Crew1Id" => "crew;merge_crew;;",
                    "BoatCaptain" => "boat_captain;copy;;","SessionType" => "session_type;copy;;",
                    "Open" => "open;copy;;",
                    "DestinationId" => "destination;use_name_if_empty;DestinationName;",
                    "WatersIdList" => "waters;use_name_if_empty;WatersNameList;",
                    "Distance" => "distance;copy_distance;;","Comments" => "comments;copy;;",
                    "LastModified" => "modified;millis_to_microtime;;"
            ],
            "efa2messages" => ["." => "messages","MessageId" => "number;copy;;",
                    "Date" => "sent;add_time;;Time","To" => "to;copy;;","From" => "from;copy;;",
                    "Subject" => "subject;copy;;","Text" => "body;copy;;","Read" => "read;copy;;",
                    "ToBeMailed" => "to_be_mailed;copy;;","ReplyTo" => "reply_to;copy;;",
                    "LastModified" => "modified;millis_to_microtime;;"
            ],
            "efa2persons" => ["." => "persons","Id" => "uuid;copy;;","FirstName" => "first_name;copy;;",
                    "LastName" => "last_name;copy;;","NameAffix" => "name_affix;copy;;",
                    "Title" => "title;copy;;","Gender" => "gender;copy;;","Birthday" => "birthday;copy;;",
                    "Association" => "association;copy;;","StatusId" => "status;copy;;",
                    "AddressStreet" => "addr_street;copy;;","AddressAdditional" => "addr_additional;copy;;",
                    "AddressCity" => "addr_city;copy;;","AddressZip" => "addr_zip;copy;;",
                    "AddressCountry" => "addr_country;copy;;","Email" => "email;copy;;",
                    "MembershipNo" => "membership_number;copy;;","ExternalId" => "external_id;copy;;",
                    "Disability" => "disability;copy;;","DefaultBoatId" => "default_boat_uuid;copy;;",
                    "ExcludeFromStatistics" => "exclude_from_statistics;copy;;",
                    "ExcludeFromCompetition" => "exclude_from_competition;copy;;",
                    "ExcludeFromClubwork" => "exclude_from_clubwork;copy;;",
                    "BoatUsageBan" => "boat_usage_ban;copy;;","EfbId" => "efb_id;copy;;",
                    "FreeUse1" => "free_use_1;copy;;","FreeUse2" => "free_use_2;copy;;",
                    "FreeUse3" => "free_use_3;copy;;","ValidFrom" => "valid_from;millis_to_microtime;;",
                    "InvalidFrom" => "invalid_from;millis_to_microtime;;","Invisible" => "invisible;copy;;",
                    "Deleted" => "deleted;copy;;","LastModified" => "modified;millis_to_microtime;;"
            ],
            "efa2status" => ["." => "status","Id" => "uuid;copy;;","Name" => "name;copy;;",
                    "Type" => "type;copy;;"
            ],
            "efa2waters" => ["." => "waters","Id" => "uuid;copy;;","Name" => "name;copy;;",
                    "Details" => "details;copy;;","EfbId" => "efb_id;copy;;",
                    "LastModified" => "modified;copy;;"
            ],
            "efaCloudUsers" => ["." => "persons","Vorname" => "first_name;copy_as_update;;",
                    "Nachname" => "last_name;copy_as_update;;","EMail" => "email;copy_as_update;;",
                    "efaCloudUserID" => "user_id;copy_as_update;;","Rolle" => "Rolle;copy_as_update;;",
                    "Subskriptionen" => "Subskriptionen;copy_as_update;;",
                    "Workflows" => "Workflows;copy_as_update;;",
                    "Concessions" => "Concessions;copy_as_update;;",
                    "Passwort_Hash" => "Passwort_Hash;copy_as_update;;",
                    "PersonId" => "uuid;id_at_other_table;;",
                    "LastModified" => "modified;millis_to_microtime;;"
            ]
    
    ];

    /**
     * for book name insertion
     */
    private static $efa_extra_to_dilbo_tables = [
            "efa2logbook" => ["logbook.logbookname;copy_bookname;;"
            ],"efa2clubwork" => ["workbook.workbookname;copy_bookname;;"
            ]
    ];

    /**
     * The result cache for a record mapping
     */
    private static $dilbo_record;

    /**
     * Errors, warnings and info log for mapping.
     */
    private static $errors = [];

    private static $warnings = [];

    private static $info = [];

    /**
     * clear the self:$errors and self:$findings
     */
    public static function clear_findings ()
    {
        self::$errors = [];
        self::$warnings = [];
        self::$info = [];
    }

    /**
     * Get the first variant's "TypeType" value. This is assumed to be identical for all variants
     * 
     * @param array $efa2boats_record
     *            the record to get the asset subtype from
     * @param Tfyh_config_item $asset_subtypes_branch
     *            the branch to put the subtype to. set null or omit to skip adding.
     * @return String the boat's first variant's TypeType value.
     */
    public static function compile_boat_subtypes (array $efa2boats_record, 
            Tfyh_config_item $asset_subtypes_branch = null)
    {
        // extract the first variants construction
        $subtype_name = explode(";", 
                (isset($efa2boats_record["TypeType"])) ? $efa2boats_record["TypeType"] : "OTHER")[0];
        if (! is_null($asset_subtypes_branch)) {
            $subtype = $asset_subtypes_branch->get_child($subtype_name, ".appObjects.CatalogItem");
            // text_local_name is a dumb default. Edit manually. Most are covred by the application
            // defaults.
            $subtype->set_descriptor(["text_local_name" => strtolower($subtype_name)
            ]);
        }
        return $subtype_name;
    }

    /**
     * Compile a set of boat variants as are available in this boat record and append it o the configuration.
     * 
     * @param array $efa2boats_record
     *            the boats record to use
     * @param Tfyh_config_item $boat_variants_branch
     *            the configuration branch with the asset item type catalog. Set null or omit to skip
     *            configuration adaptation.
     */
    public static function compile_boat_variants (array $efa2boats_record, 
            Tfyh_config_item $boat_variants_branch = null)
    {
        $variant_indices = explode(";", $efa2boats_record["TypeVariant"]);
        $bv_names = [];
        for ($v = 0; $v < count($variant_indices); $v ++) {
            // extract the values
            $coxing = explode(";", 
                    (isset($efa2boats_record["TypeCoxing"])) ? $efa2boats_record["TypeCoxing"] : "OTHER")[$v];
            $seating = explode(";", 
                    (isset($efa2boats_record["TypeSeats"])) ? $efa2boats_record["TypeSeats"] : "OTHER")[$v];
            $places = intval($seating) + ((strcasecmp($coxing, "COXED") == 0) ? 1 : 0);
            $rigging = explode(";", 
                    (isset($efa2boats_record["TypeRigging"])) ? $efa2boats_record["TypeRigging"] : "OTHER")[$v];
            // create a boat variant
            $bv_name = $coxing . "_" . $seating . "_" . $rigging;
            $bv_names[$variant_indices[$v]] = $bv_name;
            if (! is_null($boat_variants_branch)) {
                $variant = $boat_variants_branch->get_child($bv_name, ".appObjects.BoatVariant");
                // text_local_name is a dumb default. Edit manually. Most are covred by the application
                // defaults.
                $variant->set_descriptor(
                        ["text_local_name" => str_replace("_", " ", strtolower($bv_name))
                        ]);
                // set the values
                $variant->get_child("coxing")->set_descriptor(["value_current" => $coxing
                ]);
                $variant->get_child("seating")->set_descriptor(["value_current" => $seating
                ]);
                $variant->get_child("places")->set_descriptor(["value_current" => $places
                ]);
                $variant->get_child("rigging")->set_descriptor(["value_current" => $rigging
                ]);
            }
        }
        return $bv_names;
    }

    /**
     * Set the self::$validation_error according to the reason of validation failure
     * 
     * @param int $type_of_finding.
     *            0 = info, 1 = warning, 2 = error
     * @param String $violated_value            
     */
    private static function add_finding (int $type_of_finding, String $message)
    {
        if ($type_of_finding == 0)
            self::$info[] = $message;
        if ($type_of_finding == 1)
            self::$warnings[] = $message;
        if ($type_of_finding == 2)
            self::$errors[] = $message;
    }

    /**
     * Simple getter
     */
    public static function get_errors ()
    {
        return self::$errors;
    }

    /**
     * Simple getter
     */
    public static function get_warnings ()
    {
        return self::$warnings;
    }

    /**
     * Simple getter
     */
    public static function get_info ()
    {
        return self::$info;
    }

    /**
     * Map all efaCloud parameter values to the respective dilbo configuration fields
     * 
     * @param array $efaCloud_cfg_app
     *            the array of efaCloud configuration parameters
     * @param Tfyh_config $config
     *            the dilbo configuration tree
     */
    public static function map_efaCloud_config (array $efaCloud_cfg_app, Tfyh_config $config)
    {
        foreach (self::$efaCloud_to_dilbo_config as $efaCloud_name => $dilbo_path) {
            if ($dilbo_path !== false) {
                $target = $config->get_by_path($dilbo_path);
                if (! is_null($target)) {
                    $value = Tfyh_data::parse($efaCloud_cfg_app[$efaCloud_name], $target->get_type());
                    $target->set_descriptor(["value_current" => $value
                    ]);
                }
            }
        }
    }

    /**
     * Remove the year part of a book name and replace it by a default, if the remainder is empty.
     * 
     * @param String $bookname            
     */
    public static function normalize_bookname (String $bookname, String $efa2tablename)
    {
        $normalized = trim(str_replace("_", " ", $bookname));
        $ystart = strpos($normalized, "2");
        if (($ystart !== false) && (mb_strlen($normalized) >= ($ystart + 4)) &&
                 is_numeric(mb_substr($normalized, $ystart, 4)) &&
                 (intval(mb_substr($normalized, $ystart, 4)) >= 2000) &&
                 (intval(mb_substr($normalized, $ystart, 4)) <= 2999)) {
            $normalized = str_replace(mb_substr($normalized, $ystart, 4), "", $normalized);
            if (strlen($normalized) < 2)
                $normalized = substr($efa2tablename, 4);
        }
        return $normalized;
    }

    /**
     * Get the target table in dilbo for an efa/efaCloud tablename.
     * 
     * @param String $efa_table
     *            the efa/efaCloud tablename.
     * @return the respective dilbo table name, or false, if this table has no mapping (i.e. is not used in
     *         dilbo).
     */
    public static function dilbo_table (String $efa_table)
    {
        if (! isset(self::$efa_efaCloud_to_dilbo_mapping[$efa_table]))
            return false;
        return self::$efa_efaCloud_to_dilbo_mapping[$efa_table]["."];
    }

    /**
     * Apply a policy as provided in self::$efa_efaCloud_to_dilbo_tables and self::$efa_extra_to_dilbo_tables
     * to add one or more fields to self::$dilbo_record. Findings are reported. In case of failure of the
     * target record, self::$dilbo_record will be false.
     * 
     * @param String $efa_tablename
     *            the name of the efa table to which this policy belongs
     * @param String $efa_bookname
     *            the name of the book for logbooks and clubwork books
     * @param String $efa_column
     *            the column of the efa table to which this policy belongs (null for extra fields)
     * @param array $efa_record_as_strings
     *            the record as associative array, all values as String, not parsed.
     * @param String $dilbo_table
     *            the table into which this data field shall be mapped.
     * @param String $policy
     *            the policy to apply. Csv-string, see self::$efa_efaCloud_to_dilbo_mapping for details.
     * @param Tfyh_socket $socket
     *            * The data base connection socket.
     */
    private static function apply_policy (String $efa_tablename, String $efa_bookname, 
            String $efa_column = null, array $efa_record_as_strings, String $dilbo_table, String $policy, 
            Tfyh_socket $socket)
    {
        $rule = explode(";", $policy);
        $dilbo_column = $rule[0];
        $value = (is_null($efa_column)) ? null : $efa_record_as_strings[$efa_column];
        $if_empty = "";
        if ((strlen($rule[2]) > 0) && (strlen($value) == 0)) {
            if (isset($efa_record_as_strings[$rule[2]]))
                $if_empty = $efa_record_as_strings[$rule[2]];
            else
                self::add_finding(0, i("7zx0jm|empty Id and Name for °%...", $rule[2]));
        }
        $add = ((strlen($rule[3]) > 0) && isset($efa_record_as_strings[$rule[3]])) ? $efa_record_as_strings[$rule[3]] : "";
        
        // now execute handling as defined in self::$efa_efaCloud_to_dilbo_handlings.
        $handling = array_search($rule[1], self::$efa_efaCloud_to_dilbo_handlings);
        if ($handling === false)
            // === undefined (beware of 0/false distinction)
            // indicates a wrong definition of self::$efa_efaCloud_to_dilbo_mapping.
            self::add_finding(1, 
                    i("m7sKC8|undefined handling °%1° ...", $rule[1], $efa_tablename, $efa_column));
        elseif ($handling === 0)
            // === "add_time"
            self::$dilbo_record[$dilbo_column] = trim(
                    ((strlen($value) > 0) ? $value : $if_empty) . " " . $add);
        elseif (($handling === 1) || ($handling === 2))
            // === "copy", "copy_as_update"
            self::$dilbo_record[$dilbo_column] = $value;
        elseif ($handling === 3)
            // === "copy_bookname"
            self::$dilbo_record[$dilbo_column] = self::normalize_bookname($efa_bookname, $efa_tablename);
        elseif ($handling === 4) {
            // === "copy_distance"
            $distance = 0;
            if (strpos($value, "km") === false) {
                if ((strpos($value, "m") !== false) && is_numeric(trim(str_replace("m", "", $value))))
                    $distance = floatval(trim(str_replace("m", "", $value))) / 1000;
                else {
                    self::add_finding(2, 
                            i("dvGVan|missing unit °km° or °m°...", $value, $efa_tablename, $efa_column));
                    $distance = floatval(trim($value));
                }
            } else
                $distance = floatval(trim(str_replace("km", "", $value)));
            self::$dilbo_record[$dilbo_column] = $distance;
        } elseif ($handling === 5) {
            // === "copy_list"
            // replace list separator. Affects allowed_group_uuids, destination_areas, waters_uuids, and
            // member_uuids. Elements of list will never contain a ";"
            self::$dilbo_record[$dilbo_column] = str_replace(";", "|", $value);
        } elseif ($handling === 6) {
            // === "create_boat_variants"
            $bv_names = self::compile_boat_variants($efa_record_as_strings, null);
            $subtype_name = self::compile_boat_subtypes($efa_record_as_strings, null);
            $variant_indices = "";
            $variant_options = "";
            foreach ($bv_names as $index => $option) {
                $variant_indices .= "|" . $index;
                $variant_options .= "|" . $option;
            }
            if (strlen($variant_indices) > 0) {
                $variant_indices = substr($variant_indices, 1);
                $variant_options = substr($variant_options, 1);
            }
            self::$dilbo_record[$dilbo_column] = $variant_indices;
            self::$dilbo_record["asset_subtype"] = $subtype_name;
            self::$dilbo_record["variant_options"] = $variant_options;
        } elseif ($handling === 7) {
            // === "id_at_other_table"
            // find the target record and add the uid to ensure an update instead of an insert
            if (strlen($value) > 0) {
                // a referencing id is provided
                // The lookup table is always versionized, use last valid records only.
                $matching = [$dilbo_column => $value
                ];
                $target_records = $socket->find_records_sorted_matched($dilbo_table, $matching, 1, "=", 
                        "invalid_from", false);
                if ($target_records === false) {
                    // reference could not be resolved. Refuse insert
                    self::add_finding(2, i("QR0NE4|Failed to find record wi...", $value, $dilbo_table));
                    self::$dilbo_record = false;
                } else {
                    // add the uid of the record to update
                    self::$dilbo_record["uid"] = $target_records[0]["uid"];
                }
            } else {
                // no reference is provided. No lookup, just insert a new record
                list ($usec, $sec) = explode(" ", microtime());
                self::$dilbo_record["valid_from"] = $sec;
                self::$dilbo_record["invalid_from"] = 9.22337E+15; // in seconds, compare max. of millis
                if (! isset(self::$dilbo_record["uuid"]))
                    self::$dilbo_record["uuid"] = Tfyh_toolbox::static_create_GUIDv4();
            }
        } elseif ($handling === 8) {
            // <=== map_damage_numbers. efa damage numbers are not unique, they are therfore renumbered and
            // the efa number is chached.
            self::$dilbo_record["efa_number"] = $value;
            $reference = $value . "." . $efa_record_as_strings["BoatId"];
            $new_number = $_SESSION["efaImport"]["map_damage_numbers"][$reference];
            self::$dilbo_record[$dilbo_column] = strval($new_number);
        } elseif ($handling === 9) {
            // === "merge_crew"
            $crew = "";
            $coxed = 0; // this is boolean false in SQL tinyint representation. Will no be parsed and
                        // formatted.
            if (isset($efa_record_as_strings["CoxId"]) && (strlen($efa_record_as_strings["CoxId"]) > 0)) {
                $crew .= "|" . $efa_record_as_strings["CoxId"];
                $coxed = 1;
            } elseif (isset($efa_record_as_strings["CoxName"]) &&
                     (strlen($efa_record_as_strings["CoxName"]) > 0)) {
                $crew .= "|" . $efa_record_as_strings["CoxName"];
                $coxed = 1;
            }
            for ($i = 1; $i <= 24; $i ++) {
                $id_name = "Crew" . $i . "Id";
                $name_name = "Crew" . $i . "Name";
                if (isset($efa_record_as_strings[$id_name]) && (strlen($efa_record_as_strings[$id_name]) > 0)) {
                    $crew .= "|" . $efa_record_as_strings[$id_name];
                } elseif (isset($efa_record_as_strings[$name_name]) &&
                         (strlen($efa_record_as_strings[$name_name]) > 0)) {
                    $crew .= "|" . $efa_record_as_strings[$name_name];
                }
            }
            self::$dilbo_record[$dilbo_column] = (strlen($crew) > 0) ? substr($crew, 1) : "";
            self::$dilbo_record["coxed"] = $coxed;
        } elseif ($handling === 10) {
            // === "millis_to_microtime"
            // eternity in efa is 9223372036854775807 (19 characters) = 9.2E+18. Conversion to float will
            // ensure that this is always the greatest value and search for the last valid one will return
            // this value first
            self::$dilbo_record[$dilbo_column] = Tfyh_toolbox::timef($value);
        } elseif ($handling === 11) {
            // === "skip"
            // fields of that type must not be part of the $efa_efaCloud_to_dilbo_tables list.
            self::add_finding(1, 
                    "skipped irrelevant field. Please correct Dilbo_config::\$efa_efaCloud_to_dilbo_tables list.");
        } elseif ($handling === 12) {
            // "use_name_if_empty"
            self::$dilbo_record[$dilbo_column] = ((strlen($value) > 0) ? $value : $if_empty);
        } elseif ($handling === 13) {
            // "use_name_list_if_empty"
            self::$dilbo_record[$dilbo_column] = ((strlen($value) > 0) ? $value : $if_empty);
            // replace list separator. If names of the list contain a ";", the effecting list change is
            // ignored (this rule affects the waters name list only).
            self::$dilbo_record[$dilbo_column] = str_replace(";", "|", self::$dilbo_record[$dilbo_column]);
        }
    }

    /**
     * Map an efa record to a dilbo record. This requires full configuration initialization. the result may be
     * more than one record, if a record is split into two, e.g.
     * 
     * @param String $efa_tablename
     *            the name of the table to which the efa record belongs
     * @param String $efa_bookname
     *            the name of the book to which the efa record belongs. This is only used for logbooks and
     *            clubwork books.
     * @param array $efa_record_as_strings
     *            the record as associative array, all values as String, not parsed.
     * @param Tfyh_socket $socket
     *            * The data base connection socket.
     * @return array|boolean the dilbo record as Strings. Use it for update instead of insert, if it contains
     *         a uid. In case of failures, false is returned.
     */
    public static function map_record_efa_to_dilbo (String $efa_tablename, String $efa_bookname, 
            array $efa_record_as_strings, Tfyh_socket $socket)
    {
        // clear result cache
        self::$dilbo_record = [];
        // declare fields which shall be replaced, if empty, but are not existing, thus not mepped.
        if (isset(self::$efa_efaCloud_to_dilbo_mapping[$efa_tablename])) {
            foreach (self::$efa_efaCloud_to_dilbo_mapping[$efa_tablename] as $efa_column => $policy) {
                $rule = explode(";", $policy);
                if (isset($rule[2]) && (strlen($rule[2]) > 0) && ! isset($efa_record_as_strings[$efa_column]))
                    $efa_record_as_strings[$efa_column] = "";
            }
        }
        
        // map existing fields
        if (isset(self::$efa_efaCloud_to_dilbo_mapping[$efa_tablename])) {
            $dilbo_table = self::$efa_efaCloud_to_dilbo_mapping[$efa_tablename]["."];
            // map existing fields
            foreach ($efa_record_as_strings as $efa_column => $value) {
                if (isset(self::$efa_efaCloud_to_dilbo_mapping[$efa_tablename][$efa_column])) {
                    Dilbo_config::apply_policy($efa_tablename, $efa_bookname, $efa_column, 
                            $efa_record_as_strings, $dilbo_table, 
                            self::$efa_efaCloud_to_dilbo_mapping[$efa_tablename][$efa_column], $socket);
                    if (self::$dilbo_record === false)
                        return false;
                }
            }
        }
        
        // add extra fields
        if (isset(self::$efa_extra_to_dilbo_tables[$efa_tablename])) {
            foreach (self::$efa_extra_to_dilbo_tables[$efa_tablename] as $table_and_policy) {
                $dilbo_table = explode(".", $table_and_policy)[0];
                $policy = explode(".", $table_and_policy)[1];
                Dilbo_config::apply_policy($efa_tablename, $efa_bookname, null, $efa_record_as_strings, 
                        $dilbo_table, $policy, $socket);
            }
        }
        // return result
        return self::$dilbo_record;
    }

    /**
     * Default lists at first install or efa import
     */
    public static function add_type_list_defaults (Tfyh_toolbox $toolbox)
    {
        $dilbo_defaults = [
                ".appSettings.catalogs.location_types" => ["BOATHOUSE" => i("Jo6uqI|boathouse"),
                        "CLUBHOUSE" => i("UFw212|club house"),"OTHER" => i("MWRaEb|other")
                ],
                ".appSettings.catalogs.asset_status" => ["AVAILABLE" => i("Isll4Y|available"),
                        "ONTHEWATER" => i("Kz84tD|on the water"),"BOOKED" => i("x8z9wo|booked"),
                        "NOTAVAILABLE" => i("nqOm9O|not available")
                ],
                ".appSettings.catalogs.damage_severities" => ["FULLYUSEABLE" => i("QTuLpA|fully usable"),
                        "LIMITEDUSEABLE" => i("tNjoKN|limited usable"),"NOTUSEABLE" => i("e5qiYY|not usable")
                ],".appSettings.catalogs.destination_areas" => ["OTHER" => i("K9vQXC|other")
                ],
                ".appSettings.catalogs.message_destinations" => ["ADMIN" => i("1VN7Pt|administrator"),
                        "BOATM" => i("4Je6Ur|boat keeper")
                ],
                ".appSettings.catalogs.languages" => ["en" => i("vnSBNn|English"),
                        "de" => i("tqnYyt|deutsch"),"fr" => i("WDb22v|Franais"),"it" => i("dQeWAo|italiano"),
                        "nl" => i("mb1C22|Nederlands")
                ],
                ".appSettings.catalogs.timezones" => ["CET" => i("tYh9jK|Central European Time")
                ],
                ".appSettings.catalogs.asset_subtypes" => ["AGIG" => "A-Gig","BGIG" => "B-Gig",
                        "CGIG" => "C-Gig","DGIG" => "D-Gig","EGIG" => "E-Gig","BARQUE" => "barque",
                        "CHURCHBOAT" => "church boat","ERG" => "Ergo","INRIGGER" => "Inrigger",
                        "MOTORBOAT" => "motor boat","RACING" => "racing","TRIMMY" => "trimmy",
                        "WHERRY" => "wherry","SEAKAYAK" => "Seakayak","RACINGKAYAK" => "Racing Kayak",
                        "WHITEWATERKAYAK" => "White Water Kayak",
                        "CANADIANTOURINGCANOE" => "Canadian Touring Canoe","POLOBOAT" => "Polo Boat",
                        "FOLDINGCANOE" => "Folding Canoe","CANADIANTEAMCANOE" => "Canadian team canoe",
                        "DRAGONBOAT" => "Dragonboat","OTHER" => "other"
                ],
                ".appSettings.catalogs.boat_coxing" => ["COXED" => "with cox","COXLESS" => "without cox",
                        "OTHER" => "other"
                ],
                ".appSettings.catalogs.person_gender" => ["FEMALE" => "female","MALE" => "male"
                ],
                ".appSettings.catalogs.boat_seating" => ["1" => "One","2" => "double sweep",
                        "2X" => "double scull","3" => "threes","4" => "fours","4X" => "double fours",
                        "5" => "fives","6" => "sixes","6X" => "double sixes","8" => "eights",
                        "8X" => "dozble eights","OTHER" => "other"
                ],
                ".appSettings.catalogs.boat_rigging" => ["SCULL" => "scull","SWEEP" => "sweep",
                        "OTHER" => "other"
                ],
                ".appSettings.catalogs.session_types" => ["ERG" => "Ergo","LATEENTRY" => "late entry",
                        "MOTORBOAT" => "motor boat","NORMAL" => "normal session","REGATTA" => "regatta",
                        "TRAININGCAMP" => "training camp"
                ],
                ".appSettings.catalogs.person_status" => ["GUEST" => "Gast","OTHER" => "andere"
                ],
                ".appSettings.catalogs.reservation_types" => ["ONETIME" => "one time",
                        "WEEKLY" => "weekly"
                ]
        ];
        foreach ($dilbo_defaults as $path => $list) {
            foreach ($list as $name => $text_local_name) {
                $cfg = $toolbox->config->get_by_path($path);
                if (! isset($cfg))
                    echo $path;
                if (! $toolbox->config->get_by_path($path)->has_child($name)) {
                    $new_child = $toolbox->config->get_by_path($path)->get_child($name, 
                            ".appObjects.CatalogItem", null, $text_local_name);
                }
            }
        }
    }

    /**
     *
     * @return the start of the sports year which is the current sports year based on the sports_year_start
     *         configuration parameter as time (seconds since 1970).
     * @param Tfyh_config $config
     *            the toolbox's configuration
     * @param int $so_many_years_ago
     *            the number of years back, default is 0
     * @return number the system time for the sports years start
     */
    public static function get_sports_year_start (Tfyh_config $config, int $so_many_years_ago = 0)
    {
        $config->include_settings("clubSettings");
        $sysm = $config->get_by_path(".clubSettings.habits.sports_year_start")->get_value_as_string("csv");
        $sports_year_start_month = (strlen($sysm) == 1) ? "0" . $sysm : $sysm;
        $sports_year_start_this_year = strtotime(date("Y") . "-" . $sports_year_start_month . "-01");
        if ($sports_year_start_this_year > time())
            // the current sports year started last calendar year
            $so_many_years_ago ++;
        $sports_year_start_years_ago = strtotime(
                (intval(date("Y")) - $so_many_years_ago) . "-" . $sports_year_start_month . "-01");
        return $sports_year_start_years_ago;
    }
}
