<?php
/**  
 *
 *       dilbo - digital logbook for Rowing and Canoeing
 *       -----------------------------------------------
 *       https://www.dilbo.org
 *
 * Copyright  2023-2024  Martin Glade
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */


/**
 * Class file to adjust the data base layout to the version required by the current code.
 */
class Dilbo_db_trigger implements Tfyh_socket_trigger
{

    /**
     * public Constructor.
     * 
     * @param Tfyh_toolbox $toolbox
     *            application toolbox
     * @param Tfyh_socket $socket
     *            the socket to connect to the database
     */
    public function __construct (Tfyh_toolbox $toolbox, Tfyh_socket $socket)
    {
        $this->toolbox = $toolbox;
        $this->socket = $socket;
    }

    /**
     *
     * {@inheritdoc}
     *
     * @see Tfyh_socket_trigger::pre_write_transaction()
     */
    public function pre_write_transaction (int $mode, int $user_id, String $tablename, array $record)
    {
        return $this->add_trigger_fields($mode, $user_id, $tablename, $record);
    }

    /**
     * add all system fields to the record. The "owner" and the "created" types will only be added for insert.
     * Set the system fields in the settings_tfyh file, in the module record & db_layout; system fields are
     * type=name, e.g uuid=Id records.uid=uid.
     * 
     * @param int $mode
     *            the mode of modification, i.e. 1 = ins(ert), 2 = upd(ate), 3 = del(ete)
     * @param int $user_id
     *            the user of the transaction. Will be added as owner field.
     * @param String $tablename
     *            the tablen to whicht hte record belongs
     * @param array $record
     *            the record to which the system fields shall be added
     * @return array|bool the record, if modified, else false.
     */
    private function add_trigger_fields (int $mode, int $user_id, String $tablename, array $record)
    {
        $modified = false;
        // special case autoincrement logbook or workbook number
        if ($mode == 1) {
            if (strcmp($tablename, "logbook") == 0)
                $matching = ["logbookname" => $record["logbookname"]
                ];
            elseif (strcmp($tablename, "workbook") == 0)
                $matching = ["workbookname" => $record["workbookname"]
                ];
            else 
                return true;               
            $latest_entry = $this->socket->find_records_sorted_matched($tablename, $matching, 1, "=", 
                    "number", false);
            $next_entry_number = ($latest_entry === false) ? 1 : intval($latest_entry["number"]);
            $provided_entry_number = intval($record["number"]);
            if ($provided_entry_number < $next_entry_number) {
                $record["number"] = $next_entry_number;
                return $record;
            }
        }
        return true;
    }
}
    
