<?php
/**  
 *
 *       dilbo - digital logbook for Rowing and Canoeing
 *       -----------------------------------------------
 *       https://www.dilbo.org
 *
 * Copyright  2023-2024  Martin Glade
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

/**
 * snippet to execute after upgrade to this version.
 */
// toolbox und socket must be reinitialzid to reload the settings_tfyh application configuration, which may
// have changed. However, the classes cannot be redeclared - we have to live with the old ones for now.
echo "Initialisierung der neuen Version ... ";
// though the class cannot be redeclared, the objects can be reinstantiated, needed to reload all
// configuration
$toolbox = new Tfyh_toolbox();
$socket = new Tfyh_socket($toolbox);
// ===== Basic data base access checks
echo "Datenbankzugang ...";
$connected = $socket->open_socket();
if ($connected)
    echo "ok ...";
else {
    echo "da klappt was nicht. Abbruch.";
    exit();
}

// ===== restart session
if (method_exists($toolbox->app_sessions,'web_session_start'))
    $toolbox->app_sessions->web_session_start("init_version.php", $socket);

// ===== run data base audit
include_once '../classes/tfyh_db_setup.php';
$db_setup = new Tfyh_db_setup($toolbox, $socket);
$db_ok = $db_setup->update_database_layout(true);

// ===== Reflect upgrade result
echo "<p><b>Vielen Dank für die Aktualisierung!</b><br>";
if (! $db_ok) {
    echo "Bei der Überprüfung der Datenbank wurde festgestellt, dass sie nicht komplett für diese Version vorbereitet ist.<br>";
    echo "<b>Bitte führe jetzt erst ein Datenbank-Audit durch und die dort empfohlenen Korrekturen.</b><br><br>";
    echo "<a href='../pages/db_audit.php'><input type='submit' class='formbutton' value='Audit starten'></a></p>";
} else {
    echo "Die Version " . file_get_contents("../public/version") . " ist nun betriebsbereit.";
    echo "<br>Diese Seite nun nicht neu laden, sondern als nächstes:<br><br>";
    echo "<a href='../pages/home.php'><input type='submit' class='formbutton' value='Loslegen'></a></p>";
}