<?php

/**
 * class file for the security concept generation class.
 */
class Sec_concept
{

    /**
     * The generic toolbox.
     */
    private $toolbox;

    /**
     * the Efa-tables class providung special table handling support.
     */
    private $efa_tables;

    /**
     * The data base connection socket.
     */
    private $socket;

    /**
     * The data base connection socket.
     */
    private $variables;

    /**
     * public Constructor.
     * 
     * @param Tfyh_toolbox $toolbox
     *            standard application toolbox
     * @param Efa_tables $efa_tables
     *            the efa_tables object to execute on transactions. If no execution is needed, e.g. for API
     *            testing, this can be ommitted.
     */
    public function __construct (Tfyh_toolbox $toolbox, Tfyh_socket $socket)
    {
        $this->toolbox = $toolbox;
        $this->socket = $socket;
    }

    /**
     * Prepare the variables as needed by the PDF template of the security concept.
     */
    private function prepare_variables ()
    {
        global $dfmt_d;
        $this->variables = [];
        $this->variables["CSS"] = file_get_contents("../templates/default.css");
        include_once '../classes/dilbo_config.php';
        foreach (Dilbo_config::$extra_config_files as $cfg_file => $object_if_missing)
            $this->toolbox->config->include_settings($cfg_file, $object_if_missing);
        
        // Verein
        $this->variables["Verein"] = $this->toolbox->config->get_by_path(".clubSettings.clubname")->get_value();
        // Druckdatum
        $this->variables["Druckdatum"] = Tfyh_data::format(Tfyh_toolbox::datetimef(), "datetime", 
                $this->toolbox->config->language_code);
        // Betriebsverantwortlich
        $this->variables["Betriebsverantwortlich"] = $this->toolbox->config->get_by_path(
                ".appSettings.operations.operations_responsible")->get_value();
        // Hoster
        $this->variables["Hoster"] = $this->toolbox->config->get_by_path(".appSettings.operations.hoster")->get_value();
        // ServerURL
        $this->variables["ServerURL"] = $_SERVER['HTTP_HOST'];
        // AVVdatum / date_of_cdp_agreement
        $this->variables["AVVdatum"] = $this->toolbox->config->get_by_path(
                ".appSettings.operations.date_of_cdp_agreement")->get_value();
        
        // dilboVersion
        $this->variables["dilboVersion"] = file_get_contents("../public/version");
        // PHPversion
        $this->variables["PHPversion"] = phpversion();
        if (strlen($this->variables["PHPversion"]) == 0)
            $this->variables["PHPversion"] = "unbekannt";
        $PHPextensions = get_loaded_extensions();
        $this->variables["PHPextensions"] = "";
        foreach ($PHPextensions as $PHPextension)
            $this->variables["PHPextensions"] .= $PHPextension . ", ";
        if (strlen($this->variables["PHPextensions"]) == 0)
            $this->variables["PHPextensions"] = "-";
        // MySQLversion
        $this->variables["MySQLversion"] = $this->socket->get_server_info();
        // dbUserKennwortlaenge
        $this->variables["dbUserKennwortlaenge"] = strval(
                mb_strlen($this->toolbox->config->get_cfg_db()["db_up"]));
        
        // max_inits_per_hour
        $this->variables["max_inits_per_hour"] = strval(
                $this->toolbox->config->settings_tfyh["init"]["max_inits_per_hour"]);
        // max_errors_per_hour
        $this->variables["max_errors_per_hour"] = strval(
                $this->toolbox->config->settings_tfyh["init"]["max_errors_per_hour"]);
        
        // accessableWebPerRole
        include_once "../classes/tfyh_menu.php";
        $menu_file_path = "../config/access/imenu";
        $audit_menu = new Tfyh_menu($menu_file_path, $this->toolbox);
        $this->variables["accessableWebPerRole"] = $audit_menu->get_allowance_profile_html($menu_file_path);
        // accessableAPIperRole
        $menu_file_path = "../config/access/api";
        $audit_menu = new Tfyh_menu($menu_file_path, $this->toolbox);
        $this->variables["accessableAPIperRole"] = $audit_menu->get_allowance_profile_html($menu_file_path);
        
        // Zugriffe Web: init, login, errors
        $activities = explode("\n", $this->toolbox->logger->get_activities_csv(14));
        $activities_table = "<table>\n";
        foreach ($activities as $activity)
            if (strcmp("_types_", substr($activity, 0, 7)) != 0)
                $activities_table .= "<tr><td>" . str_replace(";", "</td><td>", $activity) . "</td></tr>";
        $this->variables["ZugriffeWeb"] = $activities_table . "</table>";
        
        // ZugriffeAPI, cf. "../pages/home.php"
        include_once "../classes/dilbo_api.php";
        $dilbo_api = new Dilbo_api($this->toolbox, $this->socket);
        $this->variables["ZugriffeAPI"] = $dilbo_api->get_last_accesses_API(true, true);
        
        // ChangesAll
        include_once "../classes/tfyh_list.php";
        
        $changes_all = "<table><tr><td>Autor</td><td>Modifikationstyp: Anzahl Transaktionen</td></tr>";
        $changes_list = new Tfyh_list("../config/lists/verwalten", 4, "ChangeLog", $this->socket, 
                $this->toolbox);
        $changes_table = [];
        $changes_rows = $changes_list->get_rows();
        foreach ($changes_rows as $changes_row) {
            $author = $changes_row[1];
            $modification_type = explode(":", $changes_row[5], 2)[0];
            if ((strcasecmp($modification_type, "updated") != 0) &&
                     (strcasecmp($modification_type, "deleted") != 0))
                $modification_type = "inserted";
            if (! isset($changes_table[$author]))
                $changes_table[$author] = [];
            if (! isset($changes_table[$author][$modification_type]))
                $changes_table[$author][$modification_type] = 1;
            else
                $changes_table[$author][$modification_type] ++;
        }
        
        foreach ($changes_table as $author => $modification_types) {
            $changes_all .= "<tr><td>$author</td><td>";
            foreach ($modification_types as $modification_type => $count) {
                $changes_all .= $modification_type . ": " . $count . ", ";
            }
            $changes_all .= "</td></tr>";
        }
        $changes_all .= "</table>";
        $this->variables["ChangesAll"] = $changes_all;
        
        // privilegierteNutzer
        $privileged_list = new Tfyh_list("../config/lists/verwalten", 3, "Privilegierte Nutzer", $this->socket, 
                $this->toolbox);
        $privileged_rows = $privileged_list->get_rows();
        $privileged_str = "";
        foreach ($privileged_rows as $privileged_row)
            $privileged_str .= $privileged_row[4] . ": (" . $privileged_row[0] . ") " . $privileged_row[2] .
                     " " . $privileged_row[3] . "<br>";
        $this->variables["privilegierteNutzer"] = $privileged_str;
        $this->variables["auditLog"] = str_replace("\n", "<br>", 
                str_replace("  ", " &nbsp;", 
                        str_replace(">", "&gt;", 
                                str_replace("<", "&lt;", 
                                        str_replace("&", "&amp;", file_get_contents("../log/app_audit.log"))))));
    }

    /**
     * Create the security concept PDF file
     */
    public function create_HTML ()
    {
        $this->prepare_variables();
        $template = "dilbo_security_concept_" . $this->toolbox->config->language_code;
        $template_path = "../templates/" . $template . ".html";
        $template_html = file_get_contents($template_path);
        foreach ($this->variables as $key => $value) {
            $template_html = str_replace("{#" . $key . "#}", $value, $template_html);
        }
        return $template_html;
    }

    /**
     * Create the security concept PDF file
     */
    public function create_PDF ()
    {
        require_once '../classes/pdf.php';
        $pdf = new PDF($this->toolbox, $this->socket, $this->toolbox->users->user_table_name);
        $template = "dilbo_security_concept_" . $this->toolbox->config->language_code;
        $this->prepare_variables();
        
        $template_path = "../templates/" . $template . ".html";
        $template_html = file_get_contents($template_path);
        foreach ($this->variables as $key => $value) {
            $template_html = str_replace("{#" . $key . "#}", $value, $template_html);
        }
        $saved_at = $pdf->create_pdf($template, i("W2wpqV|efaCloud Security Concep..."), 0, $this->variables);
        $this->toolbox->logger->log(0, intval($this->toolbox->users->session_user["@id"]), 
                i("M4P1uK|Security concept created..."));
        return $saved_at;
    }
}
    
