<?php
/**  
 *
 *       dilbo - digital logbook for Rowing and Canoeing
 *       -----------------------------------------------
 *       https://www.dilbo.org
 *
 * Copyright  2023-2024  Martin Glade
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

/**
 * The form for upload and import of multiple data records as csv-tables. Based on the Form class, please read
 * instructions their to better understand this PHP-code part.
 * 
 * @author mgSoft
 */

// ===== initialize toolbox and socket and start session.
$user_requested_file = __FILE__;
$menu = "menu_rserver";
include_once "../classes/init.php";
include_once '../classes/tfyh_form.php';
$doStep = (isset($_GET["doStep"])) ? intval($_GET["doStep"]) : 0;
$chunk = (isset($_GET["chunk"])) ? intval($_GET["chunk"]) : 0;
$from = (isset($_GET["from"])) ? intval($_GET["from"]) : 0;

// === APPLICATION LOGIC ==============================================================

// if validation fails, the same form will be displayed anew with error messgaes
$todo = ($done == 0) ? 1 : $done;
$form_errors = "";
$info = "";
$form_layout = "../config/layouts/efa_backup_importieren";
$tmp_upload_file = "";

// add applicable extra configuration
include_once '../classes/dilbo_config.php';
foreach (Dilbo_config::$extra_config_files as $cfg_file => $object_if_missing)
    $toolbox->config->include_settings($cfg_file, null);

// ======== start with form filled in last step: check of the entered values.
if ($done > 0) {
    
    $form_filled = new Tfyh_form($form_layout, $socket, $toolbox, $done, $fs_id);
    $form_filled->read_entered(false);
    $form_errors = $form_filled->check_validity();
    $entered_data = $form_filled->get_entered();
    // cache parameters. Entered data always contained all data ever entered in a form sequence.
    $reset_db = (isset($entered_data["reset_db"])) && (strcasecmp($entered_data["reset_db"], "on") == 0);
    $efacloud_password = ((isset($entered_data["efacloud_password"])) &&
             (strlen($entered_data["efacloud_password"]) >= 8)) ? $entered_data["efacloud_password"] : false;
    
    // application logic, step by step
    if (strlen($form_errors) > 0) {
        // do nothing. This only prevents any logic to apply, if form errors
        // occured.
    } else {
        if ($done == 1) {
            file_put_contents("../log/efa2import.log", date("Y-m-d H:i:s") . ": Starting import.\n");
            // upload files
            if (($_FILES['userfile']['error'] == UPLOAD_ERR_INI_SIZE) ||
                     ($_FILES['userfile']['error'] == UPLOAD_ERR_FORM_SIZE))
                $form_errors .= i("Wy3oFO|File size limit exceeded...");
            elseif ($_FILES['userfile']['error'] == UPLOAD_ERR_PARTIAL)
                $form_errors .= i("9dSKwb|The file was only partia...");
            elseif ($_FILES['userfile']['error'] > 0)
                $form_errors .= i("OVDL2X|There was an error durin...");
            // step 1 form was filled. Values were valid
            elseif (strlen($_FILES['userfile']["name"]) < 1) {
                // Special case upload error. Userfile can not be checked after
                // being entered, must be checked
                // after upload was tried.
                $form_errors .= i("4Vpkuw|No file given. Please tr...");
            } else {
                $tmp_upload_file = file_get_contents($_FILES['userfile']['tmp_name']);
                if (! $tmp_upload_file)
                    $form_errors .= i("a5koMa|Unknown error during upl...");
                else {
                    $_SESSION["uploadFilename"] = $_FILES['userfile']["name"];
                    $efa_import_upload_dir = "../uploads/efa2import";
                    // remove any previous import. Only the very last counts.
                    if (file_exists($efa_import_upload_dir))
                        $toolbox->rrmdir($efa_import_upload_dir);
                    mkdir($efa_import_upload_dir);
                    $_SESSION["uploadPath"] = $efa_import_upload_dir . "/" . $_SESSION["uploadFilename"];
                    $result = file_put_contents($_SESSION["uploadPath"], $tmp_upload_file);
                    if ($result === false)
                        $form_errors .= i("w2qoFy|Unknown error while uplo...", $_SESSION["uploadPath"]);
                    else {
                        $_SESSION["uploadResult"] = $result;
                        $todo = $done + 1;
                    }
                }
            }
        }
        // ============== repetitive execution =========================================
        // the "$done == 2" section will be run at every little step anew.
        // The javascript modal will use the response to trigger the next step.
        // ============== repetitive execution =========================================
        if ($done == 2) {
            // configure stepwise upload efa
            $upload_steps = 2;
            $config_file_steps = 3;
            $data_file_steps = (isset($_SESSION["efaImport"]["fileListData"])) ? count(
                    $_SESSION["efaImport"]["fileListData"]) : 12;
            $efacloud_data_steps = 1;
            $word_index_steps = 1;
            // configure stepwise upload efacloud
            $projects_branch = $toolbox->config->get_by_path(".efa2import.projects");
            $include_efacloud = (! is_null($projects_branch) && ($efacloud_password !== false));
            $efa_projectname = ($include_efacloud) ? array_keys($projects_branch->get_children())[0] : false;
            
            // data import. Will be performed in fractions to enable progress display
            include_once '../classes/efa_import_export.php';
            $efa_import_export = new Efa_import_export($toolbox, $socket);
            if ($doStep > 0) {
                $remainder = 0; // this tells that the step is fully done; default
                $doneStep = 0; // no step was executed
                               // === Import preparation steps.
                if ($doStep <= $upload_steps) {
                    if ($doStep == 1) {
                        // Import upload step.
                        $efa_import_export->add_progress("<h4>" . i("jivK7Y|Backup import log") . "</h4>", 
                                false);
                        $efa_import_export->add_progress(
                                i("Tu6j0f|%1 Bytes were uploaded.", $_SESSION["uploadResult"]) . "<br>");
                        $efa_import_export->load_zip($_SESSION["uploadPath"], $reset_db);
                        $doneStep = $doStep;
                    } elseif ($doStep == 2) {
                        if ($reset_db) {
                            // Data base reset step.
                            $db_setup = new Tfyh_db_setup($toolbox, $socket);
                            $result_reset = $db_setup->init_data_base();
                            $efa_import_export->add_progress(i("IJxXoI|Data base was reset.") . "<br>");
                        } else
                            $efa_import_export->add_progress(i("h0EUOC|Data base reset was skip...") . "<br>");
                        // block completed
                        $efa_import_export->add_progress("<br>");
                        $doneStep = $doStep;
                    }
                }
                // === Import configuration steps.
                if (($doStep > $upload_steps) && ($doStep <= ($upload_steps + $config_file_steps))) {
                    if (strcasecmp($entered_data["import_config"], "on") == 0) {
                        // execute step
                        if ($doStep == 3) {
                            // import efa configuration.
                            $efa_import_export->clear_settings();
                            $efa_import_export->import_config();
                        } elseif ($doStep == 4) {
                            // compile boat variants and create a message number mapping
                            $efa_import_export->compile_boat_variants();
                            $efa_import_export->map_damage_numbers();
                        } elseif ($doStep == 5) {
                            // now check for efaCloudConfig
                            if ($include_efacloud) {
                                $efa_import_export->import_efacloud_config($efa_projectname, 
                                        $efacloud_password);
                            }
                            // block completed
                            $efa_import_export->add_progress("<br>");
                        }
                        $doneStep = $doStep;
                    } else
                        // skip block
                        $doStep = $upload_steps + $config_file_steps + 1;
                }
                // === Import efa data steps, can run in fractions.
                if (($doStep > ($upload_steps + $config_file_steps)) &&
                         ($doStep <= ($upload_steps + $config_file_steps + $data_file_steps))) {
                    if (strcasecmp($entered_data["import_data"], "on") == 0) {
                        // execute step, or step fraction. start with $file_index = 0.
                        $file_index = $doStep - ($upload_steps + $config_file_steps) - 1; // starts with 0
                        $remainder = $efa_import_export->import_data($file_index, $chunk, $from, $reset_db);
                        $doneStep = $doStep;
                        if ($doneStep == ($upload_steps + $config_file_steps + $data_file_steps)) {
                            // block completed, if not in update
                            if ($reset_db)
                                $efa_import_export->add_progress("<br>");
                        }
                    } else
                        // skip block
                        $doStep = $upload_steps + $config_file_steps + $data_file_steps + 1;
                }
                // === Import efacloud data steps, can run in fractions. start with $file_index = 0.
                if (($doStep > ($upload_steps + $config_file_steps + $data_file_steps)) && ($doStep <=
                         ($upload_steps + $config_file_steps + $data_file_steps + $efacloud_data_steps))) {
                    if ((strcasecmp($entered_data["import_data"], "on") == 0) && $include_efacloud) {
                        // execute step, or step fraction
                        $file_index = $doStep - ($upload_steps + $config_file_steps + $data_file_steps) - 1;
                        $remainder = $efa_import_export->import_efacloud_data($file_index, $chunk, $from, 
                                $reset_db, $efa_projectname, $efacloud_password);
                        $doneStep = $doStep;
                        if ($doneStep ==
                                 ($upload_steps + $config_file_steps + $data_file_steps + $efacloud_data_steps))
                            // block completed
                            $efa_import_export->add_progress("<br>");
                    } else
                        // skip block
                        $doStep = $upload_steps + $config_file_steps + $data_file_steps + $efacloud_data_steps +
                                 1;
                }
                // === Build the word index.
                if (($doStep > ($upload_steps + $config_file_steps + $data_file_steps)) && ($doStep <= ($upload_steps +
                         $config_file_steps + $data_file_steps + $efacloud_data_steps + $word_index_steps))) {
                    if ((strcasecmp($entered_data["import_data"], "on") == 0)) {
                        // execute step, or step fraction
                        include_once "../classes/tfyh_word_index.php";
                        $word_index = new Tfyh_word_index($toolbox, $socket);
                        $word_index->rebuild();
                        $doneStep = $doStep;
                        // block completed
                        $efa_import_export->add_progress(i("8dPpRp|Word index built.") . "<br>");
                    } else
                        // skip block
                        $doStep = $upload_steps + $config_file_steps + $data_file_steps + $efacloud_data_steps +
                                 $word_index_steps + 1;
                }
                // respond after step execution with the step done
                if ($doneStep > 0) {
                    // add the progress text, if a step was executed
                    echo "$doneStep;$remainder;" . $efa_import_export->get_progress();
                    end_script(false);
                    exit();
                } else {
                    // return the idle keyword, if no further step was to be done
                    echo "0;0;idle";
                    end_script(false);
                    exit();
                }
            } else
                $todo = $done + 1;
        }
    }
} else {
    // done == 0, clear session parameters for a fresh start.
    unset($_SESSION["efaImport"]);
}

// ==== continue with the definition and eventually initialization of form to fill for the next step
if (isset($form_filled) && ($todo == $form_filled->get_index())) {
    // redo the 'done' form, if the $todo == $done, i. e. the validation failed.
    $form_to_fill = $form_filled;
} else {
    // if it is the start or all is fine, use a form for the coming step.
    $form_to_fill = new Tfyh_form($form_layout, $socket, $toolbox, $todo, $fs_id);
}

// === PAGE OUTPUT ===================================================================

// ===== start page output
echo file_get_contents('../config/snippets/page_01_start');
echo $menu->get_menu();
echo file_get_contents('../config/snippets/page_02_nav_to_body');

// page heading, identical for all workflow steps
echo "<h3>" . i("FVkWAh|Import an efa backup arc...") . "</h3>";
echo "<p>" . i("RBI9tL|This form is import an e...") . "</p>";

if ($todo == 1) { // step 1. No texts for output
    echo $toolbox->form_errors_to_html($form_errors);
    echo $form_to_fill->get_html(true); // enable file upload
} elseif ($todo == 2) { // step 2. Files were uploaded, but import not started
    echo $toolbox->form_errors_to_html($form_errors);
    if (strlen($form_errors) == 0)
        echo "<script>var progressUrl = '../forms/efa_backup_importieren.php?fseq=" . $fs_id . "2" .
                 "';</script>";
} elseif ($todo == 3) { // step 3. fractional import of data, output at last step
    echo "<p>" . i("HBcQsm|Your backup has been imp...") . "</p>";
    echo $efa_import_export->get_progress(true);
}

// Help texts and page footer for output.
echo $form_to_fill->get_help_html();
echo "</div>";
end_script();
