<?php
/**  
 *
 *       dilbo - digital logbook for Rowing and Canoeing
 *       -----------------------------------------------
 *       https://www.dilbo.org
 *
 * Copyright  2023-2024  Martin Glade
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

/**
 * The form for user profile self service. Based on the Tfyh_form class, please read instructions their to
 * better understand this PHP-code part.
 * 
 * @author mgSoft
 */
// ===== initialize toolbox and socket and start session.
$user_requested_file = __FILE__;
// ===== page does not need an active session
include_once "../classes/init.php";
include_once '../classes/tfyh_form.php';
include_once '../classes/tfyh_word_index.php';

// === APPLICATION LOGIC ==============================================================
// if validation fails, the same form will be displayed anew with error messgaes
$todo = ($done == 0) ? 1 : $done;
$form_errors = "";
$form_layout = "../config/layouts/find_record";

// ======== Start with form filled in last step: check of the entered values.
if ($done > 0) {
    $form_filled = new Tfyh_form($form_layout, $socket, $toolbox, $done, $fs_id);
    $form_filled->read_entered();
    $form_errors = $form_filled->check_validity();
    $entered_data = $form_filled->get_entered();
    $find = $entered_data["find"];
    $uids_matching = [];
    
    // application logic, step by step
    if (strlen($form_errors) > 0) {
        // do nothing. This avoids any change, if form errors occured.
    } elseif ($done == 1) {
        if (strlen($find) < Tfyh_word_index::MIN_WORD_LENGTH)
            $form_errors = i("f8WSn3|Please provide at least ...");
        else {
            include_once '../classes/tfyh_word_index.php';
            $word_index = new Tfyh_word_index($toolbox, $socket);
            $word_index->find($find, true, true);
            $find_result_html = "<h4>" . i("rSEEGA|results for °%1°", Tfyh_word_index::to_lower_ascii($find)) . ":</h4>";
            $t = 0;
            $r = 0;
            foreach ($word_index->find_result as $tablename => $records) {
                $t ++;
                $table_config = $toolbox->config->get_by_path(".appTables." . $tablename);
                $tablename_local = (is_null($table_config)) ? $tablename : $table_config->get_descriptor_field(
                        "text_local_name");
                $find_result_html .= "<h4>" . $tablename_local . "</h4><ol>";
                foreach ($records as $uid => $record) {
                    $find_result_html .= "<li><a target='_blank' href='../pages/view_record.php?uid=" . $uid . "&table=" .
                             $tablename . "'>";
                    $find_result_html .= $record["@in_fields"] . "</a><br>" . $record["@display_as"] . "</li>";
                    $r ++;
                }
                $find_result_html .= "</ol>";
            }
            $find_result_html .= "<h4>" . i("MMIW9a|%1 records in %2 tables ...", $r, $t) . "</h4>";
            $todo = $done + 1;
        }
    }
}

// ==== continue with the definition and eventually initialization of form to fill for the next step
if (isset($form_filled) && ($todo == $form_filled->get_index())) {
    // redo the 'done' form, if the $todo == $done, i. e. the validation failed.
    $form_to_fill = $form_filled;
} else {
    // if it is the start or all is fine, use a form for the coming step.
    $form_to_fill = new Tfyh_form($form_layout, $socket, $toolbox, $todo, $fs_id);
}

// === PAGE OUTPUT ===================================================================

// ===== start page output
echo file_get_contents('../config/snippets/page_01_start');
echo $menu->get_menu();
echo file_get_contents('../config/snippets/page_02_nav_to_body');

// page heading, identical for all workflow steps
echo "<h3><br><br>" . i("XZ0WtK|Search the data base") . "</h3>";

echo $toolbox->form_errors_to_html($form_errors);
if ($todo < 2) {
    echo $form_to_fill->get_html();
    echo $form_to_fill->get_help_html();
} else
    echo $find_result_html;

echo i("01G01O|</div>");
end_script();

    
