<?php
/**  
 *
 *       dilbo - digital logbook for Rowing and Canoeing
 *       -----------------------------------------------
 *       https://www.dilbo.org
 *
 * Copyright  2023-2024  Martin Glade
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

/**
 * The form for user profile self service. Based on the Tfyh_form class, please read instructions their to
 * better understand this PHP-code part.
 * 
 * @author mgSoft
 */

// ===== initialize toolbox and socket and start session.
$user_requested_file = __FILE__;
// ===== page does not need an active session
include_once "../classes/init.php";
include_once '../classes/tfyh_form.php';
include_once '../classes/dilbo_record.php';
$dilbo_record = new Dilbo_record($toolbox, $socket);

// ===== a dummy for a password which is not the right one. Must nevertheless be a valid one to
// pass all checks further down.
$keep_password = "keuk3HVpxHASrcRn6Mpf";

// This page requires an id to be set for the user to update. If not set, or the id is 0, a new user will be
// created.
$is_new_user = false;
if (! isset($_SESSION["getps"][$fs_id]["uid"]) || (strlen($_SESSION["getps"][$fs_id]["uid"]) == 0))
    $toolbox->display_error(i("jA4eJf|Not allowed"), i("NiYUV5|You have to provide the ..."), 
            $user_requested_file);

$now = Tfyh_toolbox::timef();
if (strcasecmp($_SESSION["getps"][$fs_id]["uid"], "new") != 0) {
    // get the user record. technically there may be multiple, because of versioning.
    $matching = ["uid" => $_SESSION["getps"][$fs_id]["uid"]
    ];
    $user_to_update = $socket->find_records_sorted_matched($toolbox->users->user_table_name, $matching, 1, 
            "=", "invalid_from", false);
    if ($user_to_update === false)
        $toolbox->display_error(i("FHyizx|Not found"), 
                i("BGJkSQ|The user record for ID °...", $_SESSION["getps"][$fs_id]["uid"]), 
                $user_requested_file);
    else
        $user_to_update = $user_to_update[0];
} else {
    $is_new_user = true;
    $id_to_update = 12345678; // some hopefully free number
    $default_email = "PLEASE.CHANGE_@_THIS.ADDRESS.ORG";
    $empty_new_user = $socket->find_record_matched($toolbox->users->user_table_name, 
            [$toolbox->users->user_id_field_name => $id_to_update
            ]);
    if ($empty_new_user === false) {
        $user_to_add[$toolbox->users->user_id_field_name] = $id_to_update;
        $user_to_add[$toolbox->users->user_firstname_field_name] = "John";
        $user_to_add[$toolbox->users->user_lastname_field_name] = "Doe";
        $user_to_add[$toolbox->users->user_mail_field_name] = $default_email;
        $user_to_add["Passwort_Hash"] = $keep_password;
        $user_to_add["Rolle"] = $toolbox->users->anonymous_role;
        $user_to_add["Workflows"] = 0;
        $user_to_add["Concessions"] = 0;
        $user_to_add["valid_from"] = Tfyh_toolbox::timef();
        $user_to_add["invalid_from"] = Tfyh_data::forever_seconds;
        $user_to_add["uuid"] = Tfyh_toolbox::static_create_GUIDv4();
        $user_to_add["uid"] = Tfyh_toolbox::create_uid(6);
        $add_user_result = $dilbo_record->modify_record($user_to_add, 
                $toolbox->config->get_by_path(".appTables.persons"), 1, $toolbox->users->session_user["@id"]);
    }
    $user_to_update = $socket->find_record_matched($toolbox->users->user_table_name, 
            [$toolbox->users->user_id_field_name => $id_to_update
            ]);
    $_SESSION["getps"][$fs_id]["newid"] = $id_to_update;
}

if ($user_to_update === false)
    $toolbox->display_error(i("FHyizx|Not found"), i("BGJkSQ|The user record for ID °...", $id_to_update), 
            $user_requested_file);
$user_name_display = $user_to_update["first_name"] . " " . $user_to_update["last_name"];
$id_to_update = $user_to_update[$toolbox->users->user_id_field_name];

// === APPLICATION LOGIC ==============================================================
// if validation fails, the same form will be displayed anew with error messgaes
$todo = ($done == 0) ? 1 : $done;
$form_errors = "";
$form_layout = "../config/layouts/nutzer_aendern";

// ======== Start with form filled in last step: check of the entered values.
if ($done > 0) {
    $form_filled = new Tfyh_form($form_layout, $socket, $toolbox, $done, $fs_id);
    $form_filled->read_entered();
    $form_errors = $form_filled->check_validity();
    $entered_data = $form_filled->get_entered();
    if (strlen($entered_data["Passwort"]) == 0) {
        $form_filled->preset_value("Passwort", $keep_password);
        $form_filled->preset_value("Passwort_Wdh", $keep_password);
    }
    // application logic, step by step
    if (strlen($form_errors) > 0) {
        // do nothing. This avoids any change, if form errors occured.
    } elseif ($done == 1) {
        // get user data stored
        $nutzer_to_update_after = [];
        foreach ($user_to_update as $key => $value)
            $nutzer_to_update_after[$key] = $value;
        // Password was changed, check identity of password and repetition
        if (! strcmp($entered_data['Passwort'], $keep_password)) {
            // -------------------------------
            // password and repetition must be identical.
            if ($entered_data['Passwort'] != $entered_data['Passwort_Wdh']) {
                $form_errors .= i("xVSLCZ|The passwords must match...") . "<br>";
                $form_filled->preset_value("Passwort", $keep_password);
                $form_filled->preset_value("Passwort_Wdh", $keep_password);
            }
        }
        
        // now copy changed values, except password (will be done later)
        foreach ($entered_data as $key => $value) {
            if (strcasecmp($key, "Passwort") === 0) {
                $form_errors .= $toolbox->check_password($value);
            } else
                $nutzer_to_update_after[$key] = $value;
        }
        
        // check $nutzer_to_update_after whether all values match validity criteria
        $info = "";
        $invalid = $toolbox->users->check_user_profile($nutzer_to_update_after);
        if (strlen($invalid) > 0)
            $form_errors .= i("3vkMSJ|Error checking data:") . " " . $invalid;
        // set password to its hash value.
        elseif (strcasecmp($entered_data['Passwort_delete'], "on") == 0) {
            $nutzer_to_update_after["Passwort_Hash"] = "-";
        } elseif (strcmp($keep_password, $entered_data["Passwort"]) !== 0) {
            if ((strcasecmp($user_to_update["Rolle"], "bths") == 0) || $is_new_user)
                $nutzer_to_update_after["Passwort_Hash"] = password_hash($entered_data['Passwort'], 
                        PASSWORD_DEFAULT);
            else
                $info = i("yR7zed|The password can only be...") . "<br>";
        }
        // Password shall be deleted
        unset($nutzer_to_update_after["Passwort_Wdh"]);
        unset($nutzer_to_update_after["Passwort_delete"]);
        
        // continue, if no errors were detected
        if (strlen($form_errors) == 0) {
            $todo = $done + 1;
            $changed = false;
            $record["uid"] = $user_to_update["uid"];
            foreach ($user_to_update as $key => $value) {
                if (isset($nutzer_to_update_after[$key]) &&
                         (strcmp($user_to_update[$key], $nutzer_to_update_after[$key]) !== 0)) {
                    $changed = true;
                    if (strcmp($key, "Passwort_Hash") == 0)
                        $info .= i("b25fYT|The password has been ch...") . "<br>";
                    else
                        $info .= i("J8QheD|%1 was changed from °%2°...", $key, 
                                htmlspecialchars($user_to_update[$key]), 
                                htmlspecialchars($nutzer_to_update_after[$key])) . "<br>";
                    $record[$key] = $nutzer_to_update_after[$key];
                }
            }
        }
        
        if ($changed && ! $form_errors) {
            $record["LastModified"] = strval(time()) . "000";
            $change_result = $dilbo_record->modify_record($record, 
                    $toolbox->config->get_by_path(".appTables.persons"), 2, 
                    $toolbox->users->session_user["@id"]);
            if (strlen($change_result) > 0) {
                $form_errors .= "<br />" . i("5pH8pE|Database update command ...") . " " . $change_result;
            } else {
                $toolbox->logger->log(0, intval($nutzer_to_update_after[$toolbox->users->user_id_field_name]), 
                        "Nutzer von Verwalter(in) " . $toolbox->users->session_user["@id"] . " geändert.");
            }
        } elseif (! $form_errors) {
            $info .= i("JjCWn9|No modified data has bee...") . "</p>";
        }
        $todo = $done + 1;
    }
}

// ==== continue with the definition and eventually initialization of form to fill for the next step
if (isset($form_filled) && ($todo == $form_filled->get_index())) {
    // redo the 'done' form, if the $todo == $done, i. e. the validation failed.
    $form_to_fill = $form_filled;
} else {
    // if it is the start or all is fine, use a form for the coming step.
    $form_to_fill = new Tfyh_form($form_layout, $socket, $toolbox, $todo, $fs_id);
    if (($todo == 1)) {
        // preset values on first step.
        $user_to_update["Passwort"] = $keep_password;
        $user_to_update["Passwort_Wdh"] = $keep_password;
        $form_to_fill->preset_values($user_to_update);
        // auto-fill-in of PersonId
        if (! isset($user_to_update["uuid"]) || (strlen($user_to_update["uuid"]) < 30)) {
            // check name. If a person with the same name exists, add the PersonId.
            $matching = ["first_name" => $user_to_update["first_name"],
                    "last_name" => $user_to_update["last_name"]
            ];
            $persons_with_same_name = $socket->find_records_sorted_matched("persons", $matching, 1, "=", 
                    "InvalidFrom", false);
            if ($persons_with_same_name !== false) {
                $uuid = (count($persons_with_same_name) == 1) ? $persons_with_same_name[0]["Id"] : i(
                        "75hzYb|[several possibilities]");
                $form_to_fill->preset_value("uuid", $uuid);
            }
        }
    }
}

// === PAGE OUTPUT ===================================================================

// ===== start page output
echo file_get_contents('../config/snippets/page_01_start');
echo $menu->get_menu();
echo file_get_contents('../config/snippets/page_02_nav_to_body');

// page heading, identical for all workflow steps
if ($is_new_user) {
    echo "<h3>" . i("jwq0iS|Create a new user") . i("a6h3Ou|with the ID %1", $id_to_update) . "</h3>";
    echo "<p>" . i("GNGdV1|This form is to create a...") . "</p>";
} else {
    echo "<h3>" . i("Sni5v9|Change the profile of %1", $user_name_display) . "</h3>";
    echo "<p>" . i("Agxr6E|This form is to change a...") . "</p>";
}
echo $toolbox->form_errors_to_html($form_errors);
echo $form_to_fill->get_html();

if ($todo == 1) { // step 1. No special texts for output
    // Delete user only as delete of persons record available
    echo $form_to_fill->get_help_html();
} else {
    echo "<p><b>" . i("gvvK8s|The data change is %1per...", (($form_errors) ? i("rT8wtF|not") : "")) . "</b> ";
    echo (($form_errors) ? "" : i("9mww2x|The following changes ha...") . "<br />" . $info);
    echo "<p><a href='../pages/nutzer_profil.php?uid=" . $user_to_update["uid"] . "'>" .
             i("CHVtwP|Display changed profile ...") . "</a>";
    echo "<br /><a href='../forms/nutzer_aendern.php?uid=" . $user_to_update["uid"] . "'>" .
             i("gcFhev|Continue to change user.") . "</a></p>";
}
echo i("77gvXM|</div>");
end_script();
