<?php
/**
 *
 *       dilbo - digital logbook for Rowing and Canoeing
 *       -----------------------------------------------
 *       https://www.dilbo.org
 *
 * Copyright  2023-2024  Martin Glade
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


/**
 * A data base bootstrap script to create the server side admin tables and the first admin user.
 * 
 * @author mgSoft
 */

// ===== THIS SHALL ONLY BE USED during application configuration, then access rights shall
// be changed to "no access" - even better: or the form deleted from the site.

// ===== initialize toolbox
include_once "../classes/init_i18n.php"; // not part of init for setup, api, logout and error
include_once '../classes/tfyh_toolbox.php';

$toolbox = new Tfyh_toolbox();
// init parameters definition
$cfg = $toolbox->config->get_cfg();
load_i18n_resource($toolbox->config->language_code);

// PRELIMINARY SECURITY CHECKS
// ===== throttle to prevent from machine attacks.
$toolbox->load_throttle("inits", $toolbox->config->settings_tfyh["init"]["max_inits_per_hour"], 
        "setup_clear_db.php");

// Create PHP-wrapper socket to data base
include_once '../classes/tfyh_socket.php';
$socket = new Tfyh_socket($toolbox);
$connected = $socket->open_socket();
if ($connected !== true)
    $toolbox->display_error(i("Data base connection failed"), $connected, "../install/setup_clear_db.pbp", 
            __FILE__);

$db_name = $socket->get_db_name();

// ===== define admin user default configuration
// set defaults
$cfg_db_default["admin_vorname"] = "Alex";
$cfg_db_default["admin_nachname"] = "Admin";
$cfg_db_default["admin_id"] = "1142";
$cfg_db_default["admin_password"] = "";
$cfg_db_default["admin_password_confirm"] = "";

// ===== Form texts for admin user configuration
$cfg_db_description["admin_vorname"] = i("first name of administrator person");
$cfg_db_description["admin_nachname"] = i("last name of administrator person");
$cfg_db_description["admin_id"] = i(
        "user id for administrator person (integer number between 100 and 1000000000), PLEASE REMEMBER");
$cfg_db_description["admin_password"] = i("password for administrator account, PLEASE REMEMBER!");
$cfg_db_description["admin_password_confirm"] = i("password repeated for administrator account.");

// ===== define field format in configuration form
$cfg_db_type["admin_vorname"] = "text";
$cfg_db_type["admin_nachname"] = "text";
$cfg_db_type["admin_id"] = "text";
$cfg_db_type["admin_password"] = "password";
$cfg_db_type["admin_password_confirm"] = "password";

// === PAGE OUTPUT ===================================================================

// ===== start page output
echo file_get_contents('../config/snippets/page_01_start');
echo file_get_contents('../config/snippets/page_02_nav_to_body');
?>
<!-- START OF content -->
<div class="w3-container">
	<h3>Delete data base <?php echo $db_name; ?> and rebuild it.</h3>
</div>
<div class="w3-container">

<?php
if ((isset($_GET['done']) && intval($_GET["done"]) == 1)) {
    
    foreach ($cfg_db_default as $key => $value)
        $cfg_db_to_use[$key] = $cfg_db_default[$key];
    
    // read entered values into $cfg_to_use array.
    foreach ($cfg_db_default as $key => $value) {
        $new_value = $_POST[$key];
        if (! is_null($new_value) && (strlen($new_value) > 0))
            $cfg_db_to_use[$key] = $_POST[$key];
    }
    // check password
    if (strcmp($cfg_db_to_use["admin_password"], $cfg_db_to_use["admin_password_confirm"]) != 0) {
        ?>
	<h4>Sorry, but password and password repetition do not match.</h4>
	<p>
		You may <a href='?done=0'>try again.</a>
	</p>
<?php
        echo "</div>";
        exit(); // really exit. No test case left over.
    }
    if (strlen($toolbox->check_password($cfg_db_to_use["admin_password"])) > 0) {
        ?>
	<h4>Sorry, but the password does not comply to the password security
		rules.</h4>
	<p>
		You may <a href='?done=0'>try again</a>
	</p>
<?php
        echo "</div>";
        exit(); // really exit. No test case left over.
    }
    // set session user to selected admin, in order to be able to manipulate the data base.
    $session_user[$toolbox->users->user_firstname_field_name] = $cfg_db_to_use["admin_vorname"];
    $session_user[$toolbox->users->user_lastname_field_name] = $cfg_db_to_use["admin_nachname"];
    $session_user[$toolbox->users->user_id_field_name] = $cfg_db_to_use["admin_id"];
    $session_user["Passwort_Hash"] = password_hash($cfg_db_to_use["admin_password"], PASSWORD_DEFAULT);
    $session_user["Rolle"] = $toolbox->users->useradmin_role;
    
    // ===== create data base
    include_once '../classes/tfyh_db_setup.php';
    $tfyh_db_layout = new Tfyh_db_setup($toolbox, $socket);
    if (! isset($session_user["uuid"]))
        $session_user["uuid"] = Tfyh_toolbox::static_create_GUIDv4();
    if (! isset($session_user["uid"]))
        $session_user["uid"] = Tfyh_toolbox::create_uid(6);
    $toolbox->users->set_session_user($session_user);
    $result_bootstrap = $tfyh_db_layout->init_data_base($session_user);
    
    echo "<p>" . $result_bootstrap . "</p>";
    // Display result and next steps
    ?>
	<h3>Done.</h3>
	<p>
		The data base was deleted and rebuilt. Please <a
			href='../install/setup_finish.php'>complete the installation</a> now.
	</p>
<?php
} else {
    ?>
	<p>
		Please enter the administrator of the newly set up database here. This
		is then the only user of the new database. This user can then carry
		out all further administrative procedures in the application.<br>
	</p>
	<form action="?done=1" method="post">
		<table>

    <?php
    // Display form fields depending on the installation mode.
    foreach ($cfg_db_default as $key => $value)
        echo '<tr><td>' . $key . ':<br>' . $cfg_db_description[$key] .
                 '&nbsp;</td><td><input class="forminput" type="' . $cfg_db_type[$key] .
                 '" size="35" maxlength="250" name="' . $key . '" value="' . $value . '"></td></tr>';
    ?>
    </table>
		<br> <input class="formbutton" type="submit"
			value="Datenbank neu aufsetzen">
	</form>
	<h3>Attention: This will delete all existing data in the database "<?php echo $db_name; ?>" !!!</h3>
</div><?php
}