<?php
/**  
 *
 *       dilbo - digital logbook for Rowing and Canoeing
 *       -----------------------------------------------
 *       https://www.dilbo.org
 *
 * Copyright  2023-2024  Martin Glade
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

/**
 * An implementation of a form to define the settings of the data base access.
 * 
 * @author mgSoft
 */

// ===== THIS SHALL ONLY BE USED during application configuration, then access rights shall
// be changed to "no access" - even better: or the form deleted from the site.

// ===== initialize toolbox
include_once "../classes/init_i18n.php"; // not part of init for setup, api, logout and error
include_once '../classes/tfyh_toolbox.php';
$settings_path = "../config/settings";
$toolbox = new Tfyh_toolbox();

// PRELIMINARY SECURITY CHECKS
// ===== throttle to prevent from machine attacks.
$toolbox->load_throttle("inits", $toolbox->config->settings_tfyh["init"]["max_inits_per_hour"], 
        "setup_db_connection.php");

// Create PHP-wrapper socket to data base
include_once '../classes/tfyh_socket.php';
$socket = new Tfyh_socket($toolbox);

// ===== define default values for configuration
$cfg_db_default["db_host"] = "rdbms.hoster.xyz";
$cfg_db_default["db_name"] = "app_data_base";
$cfg_db_default["db_user"] = "db_user";
$cfg_db_default["db_up"] = "user_password";

// ===== define display text for field in configuration form
$cfg_db_description["db_host"] = i("the data base host server");
$cfg_db_description["db_name"] = i("the data base name");
$cfg_db_description["db_user"] = i("name of the technical user to directly access the data base");
$cfg_db_description["db_up"] = i("password of the technical user to directly access the data base");

// ===== define field format in configuration form
$cfg_db_type["db_host"] = "text";
$cfg_db_type["db_name"] = "text";
$cfg_db_type["db_user"] = "text";
$cfg_db_type["db_up"] = "password";

// === PAGE OUTPUT ===================================================================

// ===== start page output
echo file_get_contents('../config/snippets/page_01_start');
echo file_get_contents('../config/snippets/page_02_nav_to_body');
// page heading, identical for all workflow steps
echo "<!-- START OF content -->\n<div class='w3-container'>\n";
echo "<h3>" . i("elJM8q|Change configuration par...") . "</h3>";
echo "<p>" . i("Configure the data base connection") . "</p>";

// first set defaults
$cfg_db_to_use = $cfg_db_default;
// now overwrite defaults with existing configuration
$cfg_db_existing = $toolbox->config->get_cfg_db();
if (is_array($cfg_db_existing))
    foreach ($cfg_db_default as $key => $value)
        if (isset($cfg_db_existing[$key]))
            $cfg_db_to_use[$key] = $cfg_db_existing[$key];

// try to connect in step "done"
if ((isset($_GET['done']) && intval($_GET["done"]) == 1)) {
    
    // read entered values into $cfg_to_use array.
    foreach ($cfg_db_default as $key => $value) {
        $new_value = $_POST[$key];
        if (! is_null($new_value) && (strlen($new_value) > 0))
            $cfg_db_to_use[$key] = $_POST[$key];
    }
    
    // test database access
    $toolbox->config->set_cfg_db($cfg_db_to_use);
    $socket = new Tfyh_socket($toolbox);
    $success_db = true;
    echo "<p>" . i("Checking data base connection for: %1", $cfg_db_to_use["db_user"]) . " ... ";
    $socket = new Tfyh_socket($toolbox);
    echo i("Tfyh_socket started. Connnecting") . " ... ";
    $connect_res = $socket->open_socket();
    if ($connect_res === true)
        echo i("Success.");
    else {
        echo i("Failed - what a pity. The error is: '%1', ", $connect_res);
        $success_db = false;
    }
    echo "</p>";
    
    // store the configuration
    if ($success_db !== false) {
        // up masking
        $cfg_db_to_use["db_up"] = Tfyh_toolbox::swap_lchars($cfg_db_to_use["db_up"]);
        // add existing additional parameters
        if (is_array($cfg_db_existing))
            foreach ($cfg_db_existing as $key => $value)
                if (! isset($cfg_db_to_use[$key]))
                    $cfg_db_to_use[$key] = $cfg_db_existing[$key];
        // write to settings file
        $cfgStr = serialize($cfg_db_to_use);
        $cfgStrBase64 = base64_encode($cfgStr);
        echo "<p>" . $settings_path . '/dbSettings wird geschrieben ... ';
        $byte_cnt = file_put_contents($settings_path . "/dbSettings", $cfgStrBase64);
        echo $byte_cnt . " Bytes.</p>";
        echo "<h3>" . i("Completed successfully.") . "</h3>";
        echo "<p>" . i("The configuration of the data base access was stored.") . "</p>";
        
        $table_names = $socket->get_table_names(false);
        $has_users_table = false;
        if (count($table_names) > 0) {
            foreach ($table_names as $table_name)
                $has_users_table = $has_users_table ||
                         (strcasecmp($table_name, $toolbox->users->user_table_name) == 0);
        }
        if (! $has_users_table) {
            echo "<p>" . i("There was no table found in the data base.") . "</p>";
            echo "<a href='../install/setup_clear_db.php' target='_blank'>&rArr; " .
                     i("Please build the data base now.") . "</p>";
        } else {
            echo "<p>" . i("There is a table of users in the data base.") . "<br>";
            echo "<a href='../install/setup_finish.php'>" . i("Please terminate the installation process.") .
                     "</a></p>";
        }
    } else {
        echo "<h3>" . i("Unfortunately, something went wrong.") . "</h3>";
        echo "<p>" . i("The data base access configuration was not stored.") . "</p>";
        echo "<p><a href='?done=0'>&rArr; " . i("You may try again.") . "</a></p>";
    }
} else {
    echo "<p>" . i("Please provide the data base connection information.") . "</p>";
    echo "<form action='?done=1' method='post'>\n		<table>";
    // Display form fields depending on the installation mode.
    foreach ($cfg_db_to_use as $key => $value) {
        echo '<tr><td>' . $key . ':<br>' . $cfg_db_description[$key] .
                 '&nbsp;</td><td><input class="forminput" type="' . $cfg_db_type[$key] .
                 '" size="35" maxlength="250" name="' . $key . '" value="' . $value . '"></td></tr>' . "\n";
    }
    echo "\n    </table>\n	<br> <input class='formbutton' type='submit' value='" . i("submit") . "'>\n</form>";
}
echo "</div></body></html>";
