/**  
 *
 *       dilbo - digital logbook for Rowing and Canoeing
 *       -----------------------------------------------
 *       https://www.dilbo.org
 *
 * Copyright  2023-2024  Martin Glade
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

/**
 * Title: dilbog digital logbook for Rowing and Canoeing; Copyright (c)
 * 2020-2021 by Martin Glade Website: https://www.dilbo.org/ License: GNU
 * General Public License v2
 */

try {

	/**
	 * The full applicatiuon configuration, in particular all forms.
	 */
	var $_formTemplates = {

		editItem : 
`tags;name;label
_R;_no_input;_headline
_//_R;_no_input;<hr><b>References</b>
_//_r;~uid|column_reference|column_handling;
_//_r;*text_local_name|value_type;
_//_r;node_addable_type|node_permissions;
_//_R;_no_input;<hr><b>Values</b>;
_//_r;value_current|value_default|value_rules;
_//_r;value_min|value_max|value_size;
_//_R;_no_input;<hr><b>SQL definitions</b>
_//_r;!sql_null|sql_default|sql_indexed;
_//_r;sql_trigger_event|sql_trigger_value|;
_//_R;_no_input;<hr><b>Form input</b>
_//_R;input_label|input_type|input_value;
_//_r;input_class|input_size|input_maxlength;
_//_r;text_explanation;
_//_R;submit;Submit
_//;_no_input;`,

		createChild : 
`tags;required;name;value;label;type;class;size;maxlength
_R_1;;_no_input;;Xzhxpv|_headline;;;;
_//_r_1;;value_type;;VqquM5|data type;text;display-bold;90%;50
_//_R_1;*;name;;hPIFLc|Technical name;text;;90%;100
_//_r_1;*;text_local_name;;m1LK6t|Name to display;text;;90%;100
_//_r_1;;text_explanation;;bPubzT|Explanatory text;textarea;;90%;4
_//_R_1;;submit;0LSiZI|Submit;;submit;formbutton;;
_//;;_no_input;;;;;;`,

		editSession :
`tags;required;name;value;label;type;class;size;maxlength
_R_2;;_no_input;;OScMU6|_headline;;;;
_/_4;!;number;;AJIsxM|Trip number;text;display-bold;;
_/_4;*;uid;;YTegco|uid;text;display-bold;;
_/_4;!;logbookname;;H0hqoc|Logbook;text;display-bold;;
_//_r_4;*;startDate;;BhDs6f|Start on;date;;;
_/_4;*;startTime;;uJzUER|Start at;text;;;
_/_4;;endDate;;X2325y|End on;date;;;
_/_4;;endTime;;vnotSi|End at;text;;;
_//_r_2;*;asset_uuid;;86mTKl|Boat;text;;;
_/_2;;session_type;;PnrJ67|Type of trip;select $catalog;;;
_//_r_2;;_no_input;;;;;;
_//_r_2;;cox;;s5zjJx|Cox;text;;;
_/_2;;asset_variant;;Z0Q2Ht|asset variant;select $catalog;;;
_//_r_1;;_no_input;;jYMtPN|In the boat:;;;;
_//_r_2;;crew_1;;;text;;;
_/_2;;crew_5;;;text;;;
_//_r_2;;crew_2;;;text;;;
_/_2;;crew_6;;;text;;;
_//_r_2;;crew_3;;;text;;;
_/_2;;crew_7;;;text;;;
_//_r_2;;crew_4;;;text;;;
_/_2;;crew_8;;;text;;;
_//_R_1;;destination;;1TB0uL|Destination;text;;;
_//_r_2;*;distance;;gsC0TF|Distance (in km);text;;;
_/_2;;waters;;4dFtke|Waters;text;;;
_//_r_1;;comments;;9zuVO6|Remarks;textarea;;2;
_//_r_1;;_no_input;;;;;;
_//_r_2;;submit;jdsLwu|Submit;;submit;formbutton;;
_//;;_no_input;;;;;;`,

		reportDamage :
`tags;required;name;value;label;type;class;size;maxlength
_R_2;;_no_input;;KD2GoS|_headline;;;;
_/_4;!;number;;4i28Ys|Damage number;text;display-bold;;
_/_4;!;logbook_text;;hVfJ6e|trip number;text;display-bold;;
_//_r_2;*;asset_uuid;;LATQCn|Boat;text;;;
_/_2;*;severity;;8ThgyP|severity;select $catalog;;;
_//_r_2;*;reported_by;;WRoXOL|reported by;text;;;
_/_2;!;reported_on;;iyU5b0|reported on;text;display-bold;;
_//_r_1;*;description;;vTuJMH|description;textarea;;2;
_//_r_1;;_no_input;;;;;;
_//_r_2;;submit;JAsJeI|Report damage;;submit;formbutton;;
_//;;_no_input;;;;;;`,

		sendMessage :
`tags;required;name;value;label;type;class;size;maxlength
_R_2;;_no_input;;oPZytR|_headline;;;;
_/_4;!;number;;VKMHh2|Message number;text;display-bold;;
_/_4;!;sent;;sBF2Kv|sent;text;display-bold;;
_//_r_2;*;to;;96IyrK|to;select $catalog;;;
_//_r_2;*;from;;imJWX5|from;text;;;
_//_r_1;*;subject;;ex7gkf|subject;text;;;
_//_r_1;*;body;;JRDj96|body;textarea;;2;
_//_r_1;;_no_input;;;;;;
_//_r_2;;submit;dB4UZa|Post message;;submit;formbutton;;
_//;;_no_input;;;;;;`,

		requestReservation :
`tags;required;name;value;label;type;class;size;maxlength
_R_2;;_no_input;;yGemjk|_headline;;;;
_/_4;!;number;;ziDGIo|Reservation number;text;display-bold;;
_/_4;!;type;;ntLe03|type;text;display-bold;;
_//_r_2;*;asset_uuid;;ymS6hW|Boat;text;;;
_//_r_6;*;startDate;;uDdSRJ|Start on;date;;;
_/_6;*;startTime;;MxmSoI|Start at;select global reservationHours;;;
_/_6;*;endDate;;PNrmaK|End on;date;;;
_/_6;*;endTime;;rfLzuX|End at;select global reservationHours;;;
_//_r_2;*;person_uuid;;9qXWZ6|requested by;text;;;
_//_r_1;*;reason;;LRAfos|reason;text;;;
_//_r_1;;_no_input;;;;;;
_//_r_2;;submit;CKjbUM|Request reservation;;submit;formbutton;;
_//;;_no_input;;;;;;`,


		editPreferences :
`tags;required;name;value;label;type;class;size;maxlength
_R_1;;_no_input;;yGemjk|_headline;;;;
_//_R_2;;logbook;;XlrYdi|Logbook name;select $catalog;;;
_/_2;;workbook;;5qV7nW|Workbook name;select $catalog;;;
_//_r_2;;language;;DNnRhZ|Language;select global languageCodes;;;
_/_2;;profile;;0eyQX0|Profile;select $catalog;;;
_//_r_1;;_no_input;;;;;;
_//_r_2;;submit;o9nvp8|Save preferences;;submit;formbutton;;
_//;;_no_input;;;;;;`,


		editRecord :  // dummy form, will be replaced.
`tags;required;name;value;label;type;class;size;maxlength
_R_1;;_no_input;;yGemjk|_headline;;;;
_//_r_1;;_no_input;;;;;;
_//_r_2;;submit;RW8eum|Save record;;submit;formbutton;;
_//;;_no_input;;;;;;`,

		deleteRecord :  
`tags;required;name;value;label;type;class;size;maxlength
_R_1;;_no_input;;yGemjk|_headline;;;;
_//_r_2;;submit;fWJ6Ui|Yes, delete now;;submit;formbutton;;
_//;;_no_input;;;;;;`,

		editValidfrom :  
`tags;required;name;value;label;type;class;size;maxlength
_R_1;;_no_input;;yGemjk|_headline;;;;
_//_r_1;;valid_from_date;;FkI23f|valid from;date;;;
_//_r_2;;submit;h1DbeX|Save period start;;submit;formbutton;;
_//;;_no_input;;;;;;`,

		extendPeriod :  
`tags;required;name;value;label;type;class;size;maxlength
_R_1;;_no_input;;yGemjk|_headline;;;;
_//_r_2;;submit;faxh60|Remove period end;;submit;formbutton;;
_//;;_no_input;;;;;;`,

		limitPeriod :  
`tags;required;name;value;label;type;class;size;maxlength
_R_1;;_no_input;;yGemjk|_headline;;;;
_//_r_1;;invalid_from_date;;tolDeM|invalid from;date;;;
_//_r_2;;submit;cP2YCP|Save period end;;submit;formbutton;;
_//;;_no_input;;;;;;`,

		newPeriod :  
`tags;required;name;value;label;type;class;size;maxlength
_R_1;;_no_input;;yGemjk|_headline;;;;
_//_r_1;;valid_from_date;;81751w|valid from;date;;;
_//_r_2;;submit;1eVBQR|create new period;;submit;formbutton;;
_//;;_no_input;;;;;;`,

	};

	var $_formNames = [];
	var $_formDefs = {};
	for (var key in $_formTemplates) {
		$_formDefs[key] = TfyhToolbox.csvToObject($_formTemplates[key]);
		$_formNames.push(key);
	}
	
	/**
	 * get the start of the sports year as String, e.g. "2024-01-01". Load the
	 * club settings before calling this function.
	 */
	function $_sports_year_start() {
	    let sysm = dilboConfig.get_by_path(".clubSettings.habits.sports_year_start").get_value_as_string("csv");
	    let sports_year_start_month = (sysm.length == 1) ? "0" + sysm : sysm;
	    let thisYear = new Date().getFullYear();
	    let sports_year_start = "" + thisYear + "-" + sports_year_start_month + "-01";
	    if (new Date(sports_year_start) > new Date())
	    	return "" + (thisYear - 1) + "-" + sports_year_start_month + "-01";
	    return sports_year_start;
	}

	/**
	 * Read the user preferences. Start with the basic application defaults,
	 * replace by club defaults and finally by the users own settings.
	 */
	function identifyPreferencesToUse() {
		userPreferences = {};
		// set defaults
		preferencesDefaults = dilboConfig.get_by_path(".appObjects.Preferences");
		clubPreferencesDefaults = dilboConfig.get_by_path(".clubSettings.defaults");
		for (let field in preferencesDefaults.get_children()) {
			// ultima ratio is the above userPreferencesDefaults value
			userPreferences[field] = preferencesDefaults.get_child(field).get_descriptor_field("value_default");
			// if there is a club habit default value, use that instead
			if (clubPreferencesDefaults.has_child(field) &&
					clubPreferencesDefaults.get_child(field).get_value() &&
					(clubPreferencesDefaults.get_child(field).get_value().length > 0))  
				userPreferences[field] = clubPreferencesDefaults.get_child(field).get_value();
		}
		// get the user's own preferences
		let nvps = TfyhToolbox.splitCsvRow(user.preferences);
		for (let nvp of nvps)
			if (nvp.indexOf("=") >= 0) {
				let field = nvp.split("=")[0];
				let value_current = nvp.substring(nvp.indexOf("=") + 1);
				userPreferences[field] = value_current;
			}
		// now that the preferences are know, set the ui-preferences
		// configuration
		uiPreferencesItem = dilboConfig.get_by_path(".uiSettings.preferences");
		for (let field in uiPreferencesItem.get_children()) 
			uiPreferencesItem.get_child(field).set_descriptor( { value_current : userPreferences[field] });
	}


} catch (e) {
	dilboModal.showException(e);
}
