/**  
 *
 *       dilbo - digital logbook for Rowing and Canoeing
 *       -----------------------------------------------
 *       https://www.dilbo.org
 *
 * Copyright  2023-2024  Martin Glade
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

/**
 * Validate the input into a form field upon its change.
 */

class DilboInputValidator {

	/**
	 * Build all tables and load them from local storage
	 */
	constructor() {
	}

	/**
	 * handle layout adaption needed for a value change
	 */ 
	handleValue(input, tablename) {
		let id = $(input).attr("id");
		if (tablename == 'assets') {
			if ((id == 'cFormInput-asset_uuid') || (id == 'cFormInput-asset_variant')) {
				// validate the boat and its places
				let assetInput = $('#cFormInput-asset_uuid')[0];
				let variantInput = $('#cFormInput-asset_variant')[0];
				let asset = dilboTables.tables.assets.iNames[assetInput.value];
				let assetVariantOptions = (asset && asset.variant_options) ? asset.variant_options.split(/\|/g) : "";
				let multipleVariants = (assetVariantOptions.length > 1);
				// if no asset is provided use a coxed eights.
				let variantInputVal = (multipleVariants && variantInput && (variantInput.value.length > 0)) ? variantInput.value : assetVariantOptions[0];
				let variant = (asset && variantInput) ? dilboConfig.get_by_path(".appSettings.boat_variants").get_child(variantInputVal) : false;
				let coxed = (variant) ? (variant.get_child("coxing").get_value() == "COXED") : true;
				let places = (variant) ? variant.get_child("places").get_value() : 9;
				// show the variants selection for more than one variant options
				if (assetVariantOptions.length < 2) {
					// hide the input field for a single asset variant
					$('#cFormInput-asset_variant').parent().hide();
					$('#cFormInput-asset_variant').val((asset && asset.variant_indices) ? asset.variant_indices.split("|")[0] : 1);
				} else {
					// hide all not applicable ones
					let allVariants = dilboConfig.get_by_path(".appSettings.boat_variants").get_children();
					for (let variant in allVariants)
						if (assetVariantOptions.indexOf(variant) < 0) 
							$("#cFormInput-asset_variant option[value='" + variant + "']").remove();
					$('#cFormInput-asset_variant').parent().show();
				}
				// show a cox input field, if needed
				let coxInput = $('#cFormInput-cox')[0];
				if (coxInput) {
					if (coxed) { // for the cox toggle the full form row
						$('#cFormInput-cox').parent().show();
						places --;
					} else {
						$('#cFormInput-cox').parent().hide();
					}
				}
				// show the correct number of crew member fields
				let crewInputs = $('#cFormInput-crew_1')[0];
				if (crewInputs) {
					for (let i = 1; i <= 8; i++)
						if (i <= places) // for the crew toggle the full form
											// row
							$('#cFormInput-crew_' + i).parent().show();
						else
							$('#cFormInput-crew_' + i).parent().hide();
				}
			}
		}
		if (tablename == 'destinations') {
			if (id == 'cFormInput-destination') {
				let destination = dilboTables.tables.destinations.iNames[input.value];
				if (destination) {
					if ($('#cFormInput-waters').val().length == 0) {
						let watersIds = destination.waters_uuids.split(/\|/g);
						let watersNames = ""; 
						for (let watersId of watersIds) {
							let water = dilboTables.tables.waters.iUuids[watersId];
							watersNames += water.name + "; ";
						}
						if (watersNames.length > 1)
							watersNames = watersNames.substring(0, watersNames.length - 2);
						dilboForm.presetValue("waters", watersNames);
						$('#cFormInput-waters').val(watersNames);
					}
					if ($('#cFormInput-distance').val().length == 0) {
						dilboForm.presetValue("distance", destination.distance);
						$('#cFormInput-distance').val(destination.distance);
					}
				}
			}
		}
	}

	// check the period based validity and add the red-amber-green input field's
	// side bar.
	validateEntry(input, tablename) {
		// empty values are never checked
		if (input.value.length == 0) {
			$(input).removeClass("uuid-valid")
					.removeClass("uuid-invalid")
					.removeClass("uuid-not-found")					
					.addClass("uuid-not-checked");
			return;
		}
		// get the record and format according to the validation result.
		let record = dilboTables.tables[tablename].iNames[input.value];
		let invalidFrom = (record) ? record.invalid_from : false;
		if (invalidFrom === false)
			$(input).removeClass("uuid-valid")
					.removeClass("uuid-invalid")
					.addClass("uuid-not-found")
					.removeClass("uuid-not-checked");
		else if (invalidFrom < Math.floor(Date.now() / 1000))
			$(input).removeClass("uuid-valid")
					.addClass("uuid-invalid")
					.removeClass("uuid-not-found")					
					.removeClass("uuid-not-checked");
		else
			$(input).addClass("uuid-valid")
					.removeClass("uuid-invalid")
					.removeClass("uuid-not-found")
					.removeClass("uuid-not-checked");
		return;
	}
}