<?php
/**  
 *
 *       dilbo - digital logbook for Rowing and Canoeing
 *       -----------------------------------------------
 *       https://www.dilbo.org
 *
 * Copyright  2023-2024  Martin Glade
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

/**
 * A page to audit the complete data base.
 * 
 * @author mgSoft
 */

// ===== initialize toolbox and socket and start session.
$user_requested_file = __FILE__;
include_once "../classes/init.php";
include_once "../classes/tfyh_db_setup.php";
$db_setup = new Tfyh_db_setup($toolbox, $socket);

// ===== Improve data base status, if requested
$improve = (isset($_GET["do_improve"])) ? $_GET["do_improve"] : "";
$do_improve = (strcmp($improve, "now") == 0);
$improvements = "";

// ===== Size check
// start with the data base size in kB
// ===================================
$audit_result = "<li><b>" . i("SqtY4N|List of tables by size") . "</b></li>\n<ul>";
$table_sizes = $socket->get_table_sizes_kB();
$total_size = 0;
$table_record_count_list = "<b>" . i("NKmdC2|Size check: Tables and r...") . "</b><ul>";
$total_record_count = 0;
$total_table_count = 0;
foreach ($table_sizes as $table_name => $table_size) {
    $record_count = $socket->count_records($table_name);
    $total_record_count += $record_count;
    $total_size += intval($table_size);
    $total_table_count ++;
    $table_record_count_list .= "<li>$table_name: $record_count " . i("xhYaXI|Records") .
             ", $table_size kB]</li>";
}
$table_record_count_list .= "<li>" . i("qHF8o7|in total:") . " $total_record_count " . i("jnzy2g|Records") .
         ", $total_table_count Tabellen, $total_size kB.</li></ul>";

// ===== Layout implementation check
$verification_result = "<b>" . i("e5VOu1|Result of layout check") . "</b><ul><li>";
$db_layout_verified = $db_setup->update_database_layout(! $do_improve);
if ($db_layout_verified) {
    $verification_result .= i("m0N2tS|The layout of the data b...");
} else {
    $optimization_needed = true;
    $verification_result .= i("1Pq6VT|NOT OK.") . "</li><li>" . str_replace("\n", "</li><li>", 
            str_replace(i("DQYc1I|Verification failed"), "<b>" . i("P1TqTx|Verification failed") . "</b>", 
                    file_get_contents("../log/sys_db_audit.log")));
}
$verification_result .= "</li></ul>";

// ===== start page output
echo file_get_contents('../config/snippets/page_01_start');
echo $menu->get_menu();
echo file_get_contents('../config/snippets/page_02_nav_to_body');
// page heading, identical for all workflow steps
echo "<!-- START OF content -->\n<div class='w3-container'>\n";
echo "<h3>" . i("6ePLG9|Audit for database %1", $socket->get_db_name()) . "</h3>";
echo "<p>" . i("ImbQDK|Here is the result of th...") . "</p>";

echo $improvements;
echo $verification_result;
if ($optimization_needed)
    echo '<p><a href="?do_improve=now"><span class="formbutton">' . i("MQEsnc|Correct now - Wait - tak...") .
             '</span></a><br /><br /></p>';
echo $table_record_count_list;

echo i("hPmRBV|</div>");
end_script();
