<?php
/**  
 *
 *       dilbo - digital logbook for Rowing and Canoeing
 *       -----------------------------------------------
 *       https://www.dilbo.org
 *
 * Copyright  2023-2024  Martin Glade
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

// ===== timestamps
$php_script_started_at = microtime(true);
$debug = true;
// ===== now use the session.
if (! (isset($_GET["info"]) || (isset($_GET["set"]) && isset($_GET["listid"])))) {
    echo "404: Forbidden.";
    exit(); // really exit. No test case left over.
}

// ===== initialize toolbox & internationalization.
// Do not run init, because jsget and api access are not part of the overload
// detection.
include_once "../classes/init_i18n.php"; // usually this is included with init.php
include_once '../classes/tfyh_toolbox.php';
$toolbox = new Tfyh_toolbox();
load_i18n_resource($toolbox->config->language_code);

// ===== debug initialization
$debug = ($toolbox->config->debug_level > 0);
$jsget_debuglog = "../log/debug_jsget.log";

// ===== Create the socket similar to init.php.
include_once '../classes/tfyh_socket.php';
if (! isset($dbconnect)) {
    $socket = new Tfyh_socket($toolbox);
    $connected = $socket->open_socket();
    if ($connected !== true) {
        $script_completed = true;
        $toolbox->display_error(i("A9f60R|Database connection fail..."), $connected, "../pages/jsget.php");
    }
}
// ===== Try to open an existing session.
$session_registration_result = $toolbox->app_sessions->web_session_start("jsget.php", $socket);
if ($session_registration_result == false) {
    echo "404: Rejected.";
    exit(); // really exit. No test case left over.
}

$info = (isset($_GET["info"])) ? $_GET["info"] : "";
$set = (isset($_GET["set"])) ? $_GET["set"] : "";
$listid = (isset($_GET["listid"])) ? $_GET["listid"] : "";
$session_user = (isset($toolbox->users->session_user["@id"])) ? intval($toolbox->users->session_user["@id"]) : - 1;

// ===== special case: get current user ID. This is allowed for all, returns -1 for any anonymous request
if (strcasecmp($info, "userid") == 0) {
    echo $session_user;
    $toolbox->logger->put_timestamp($session_user, "public/jsget.php", $php_script_started_at);
    exit(); // really exit. No test case left over.
}

// ===== all other information is blocked for anonymous users.
if (! isset($toolbox->users->session_user) ||
         (strcasecmp($toolbox->users->session_user["Rolle"], $toolbox->users->anonymous_role) == 0)) {
    if (strcasecmp($info, "executionProgress") == 0)
        echo "idle";
    else
        echo "404: Forbidden.";
    exit(); // really exit. No test case left over.
}

// ===== special cases
// ..... get all last modifications
if (strcasecmp($info, "mtimes") == 0) {
    $mtimes = "";
    foreach (Tfyh_config::$mandatory_cfg_files as $cfg_file => $object_if_missing) {
        $fname = "../config/settings/" . $cfg_file;
        if (file_exists($fname))
            $mtimes .= $cfg_file . "=" . filemtime($fname) . ";";
    }
    // add all extra configuration
    include_once '../classes/dilbo_config.php';
    foreach (Dilbo_config::$extra_config_files as $cfg_file => $object_if_missing) {
        $fname = "../config/settings/" . $cfg_file;
        if (file_exists($fname))
            $mtimes .= $cfg_file . "=" . filemtime($fname) . ";";
    }
    echo substr($mtimes, 0, strlen($mtimes) - 1);
    exit(); // really exit. No test case left over.
}
// ..... cancel progress information
if (strcasecmp($info, "executionCancel") == 0) {
    $session_id = session_id();
    unlink("../log/executionProgress." . $session_id);
}
// ..... get the data type parse_as information (not part of the descriptor)
if (strcasecmp($info, "parseAs") == 0) {
    $session_id = session_id();
    $csv = "data_type;parse_as\n";
    foreach (Tfyh_data::$type_templates as $name => $template)
        $csv .= $name . ";" . Tfyh_data::parse_as($name) . "\n";
    echo $csv;
    exit(); // really exit. No test case left over.
}

// ===== return a configuration file
if ((strlen($info) > 0) && (strlen($set) == 0)) {
    // get the setting file with this name
    $config_root = $toolbox->config->get_by_path("." . $info);
    if (is_null($config_root)) {
        // not found. Add all extra configuration
        include_once '../classes/dilbo_config.php';
        foreach (Dilbo_config::$extra_config_files as $cfg_file => $object_if_missing)
            $toolbox->config->include_settings($cfg_file, $object_if_missing);
        $config_root = $toolbox->config->get_by_path("." . $info);
        if (is_null($config_root)) {
            // even not in extra configuration
            echo "502: Invalid.";
            exit(); // really exit. No test case left over.
        }
    }
    // return configuration as csv list.
    $csv = $config_root->to_string(false, "csv", 99, true);
    echo $csv;
} // return a list contents

elseif ((strlen($set) > 0) && (strlen($listid) > 0)) {
    include_once '../classes/tfyh_list.php';
    $list = new Tfyh_list("../config/lists/$set", intval($listid), "", $socket, $toolbox);
    $csv = $list->get_csv($toolbox->users->session_user);
    echo $csv;
}

$toolbox->logger->put_timestamp($session_user, "public/jsget.php", $php_script_started_at);
exit(); // really exit. No test case left over.