<?php
/**  
 *
 *       dilbo - digital logbook for Rowing and Canoeing
 *       -----------------------------------------------
 *       https://www.dilbo.org
 *
 * Copyright  2023-2024  Martin Glade
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

/**
 * Generic record display file.
 * 
 * @author mgSoft
 */

// ===== initialize toolbox and socket and start session.
$user_requested_file = __FILE__;
include_once "../classes/init.php";
include_once "../classes/tfyh_list.php";

$uid = (isset($_GET["uid"])) ? $_GET["uid"] : false;
$tablename = (isset($_GET["table"])) ? $_GET["table"] : false;
if (! $toolbox->config->get_by_path(".appTables")->has_child($tablename))
    $toolbox->display_error(i("L5LuMQ|Provided table name has ..."), 
            i("crDZRs|Page °%1° must be called...", $user_requested_file, 
                    $tablename), $user_requested_file);
$table_cfg = $toolbox->config->get_by_path(".appTables")->get_child($tablename);
$action = "editRecord";
if ($uid && $tablename) {
    $tablerow = $socket->find_record_matched($tablename, ["uid" => $uid
    ]);
    if ($tablerow == false) {
        $toDisplay = ["???" => "not found."
        ];
        $min_record = ["uid" => Tfyh_toolbox::create_uid(6),
                "uuid" => Tfyh_toolbox::static_create_GUIDv4()
        ];
        $action = "addRecord";
    } else
        $toDisplay = $tablerow;
} else {
    $toolbox->display_error(i("KjzmnU|Not allowed."), 
            i("b1CfJu|Page °%1° must be called...", $user_requested_file), $user_requested_file);
}

include_once '../classes/dilbo_record.php';
$dilbo_record = new Dilbo_record($toolbox, $socket);
$to_forward = ($tablerow === false) ? $dilbo_record->get_csv($tablename, $min_record) : $dilbo_record->get_csv(
        $tablename, $tablerow);

// remove lookup references
$toDisplay_cleansed = [];
foreach ($toDisplay as $key => $value) {
    if (strpos($key, ">") != false) {
        $column = explode(">", $key)[0];
        if ($tablerow != false)
            $toDisplay_cleansed[$column] = $tablerow[$column];
        else
            $toDisplay_cleansed[$column] = $value;
    } else
        $toDisplay_cleansed[$key] = $value;
    $toDisplay = $toDisplay_cleansed;
}

// special case: add record history, if existing
if ((! isset($toDisplay["history"]) || (strlen($toDisplay["history"]) < 3)) && isset($tablerow["history"]))
    $toDisplay["history"] = $tablerow["history"];
include_once "../classes/dilbo_uuids.php";
$dilbo_uuids = new Dilbo_uuids($toolbox, $socket);

// ===== start page output
echo file_get_contents('../config/snippets/page_01_start');
echo $menu->get_menu();
echo file_get_contents('../config/snippets/page_02_nav_to_body');

// page heading, identical for all workflow steps
echo "<h3>" .
         i("t3k8Xe|Data record display for ...", $table_cfg->get_descriptor_field("text_local_name")) .
         "</h3>";
echo "<table><tr><th>" . i("sC5sYJ|property") . "</th><th>" . i("o474TC|value") . "</th><th>" . i("yml89m|technically") .
         "</th></tr>";

$null_values = "";
$language_code = $toolbox->config->language_code;
$is_versionized = false;
foreach ($toDisplay as $key => $value) {
    if (strcmp($key, "valid_from") == 0)
        $is_versionized = true;
    $field_cfg = ($table_cfg->has_child($key)) ? $table_cfg->get_child($key) : null;
    if (is_null($field_cfg))
        echo "<tr><td>" . $key . "</td><td>" . $value . "</td><td></td></tr>\n";
    else {
        $key_locale = $field_cfg->get_descriptor_field("text_local_name");
        $field_data_type = $field_cfg->get_type();
        if (strlen($value) == 0)
            $null_values .= $key_locale . "<br>";
        elseif (strcasecmp($key, "history") == 0)
            echo "<tr><td>" . i("UhbQ08|record history") . "</td><td>" .
                     "<a href='../pages/show_history.php?table=$tablename&uid=$uid'>" .
                     i("UcNTLA|show versions") . "</a></td><td></td></tr>\n";
        elseif ((strcasecmp($field_data_type, "uuid") == 0) ||
                 (strcasecmp($field_data_type, "uuid_or_name") == 0)) {
            $values = explode("|", $value);
            $names = "";
            $matched = true;
            foreach ($values as $single_value) {
                $resolved_name = $dilbo_uuids->uuid2name($single_value);
                $matched = (strcmp($resolved_name[1], "unknown") != 0);
                $name = ($matched) ? $resolved_name[0] : $single_value;
                $names .= ", " . $name;
            }
            echo "<tr><td>" . $key_locale . "</td><td>" . substr($names, 2) . "</td><td>" . $value .
                     "</td></tr>\n";
        } elseif (strcasecmp($field_data_type, "microtime") == 0) {
            if (floatval($value) >= Tfyh_data::forever_seconds)
                echo "<tr><td>" . $key_locale . "</td><td>" . i("2xog20|never") . "</td><td>" . $value .
                         "</td></tr>\n";
            else {
                $datetime = Tfyh_toolbox::datetimef($value);
                echo "<tr><td>" . $key_locale . "</td><td>" .
                         Tfyh_data::format($datetime, "datetime", $language_code) . "</td><td>" . $value .
                         "</td></tr>\n";
            }
        } else {
            $value = Tfyh_data::format(Tfyh_data::parse($value, $field_data_type), $field_data_type, 
                    $language_code);
            echo "<tr><td>" . $key_locale . "</td><td>" . $value . "</td><td>(" . $field_data_type .
                     ")</td></tr>\n";
        }
    }
}
if (strlen($null_values) > 0)
    echo "<tr><td>" . i("eiCoTk|empty data fields") . "</td><td>" .
             mb_substr($null_values, 0, mb_strlen($null_values) - 2) . "</td><td></td></tr>\n";
echo "</table>\n";
echo "<script>var php_record_as_csv = `" . $to_forward . "`;</script>\n";
echo "<div class='w3-container'>\n<div class='w3-row' style='overflow:visible;padding-bottom:3em'>\n";

// now add the editing options.
if ($is_versionized) {
    $is_versionized_first_period = false;
    $is_versionized_last_period = false;
    $period_records = $socket->find_records_sorted_matched($tablename, ["uuid" => $toDisplay["uuid"]
    ], 20, "=", "valid_from", true);
    $cnt = count($period_records);
    if ($period_records === false)
        $is_versionized = false;
    else {
        $this_uid = $toDisplay["uid"];
        $is_versionized_first_period = (strcmp($period_records[0]["uid"], $this_uid) == 0);
        $is_versionized_last_period = (strcmp($period_records[$cnt - 1]["uid"], $this_uid) == 0);
    }
}
if ($menu->is_allowed_menu_item("../js/dilboFormHandler/editRecord", $toolbox->users->session_user)) {
    echo "<h4>" . i("2gTQi0|Editing options") . ":</h4><ol>";
    echo "<li><a href='#' onclick='handleEvent(\"" . $action . "_" . $tablename . "_" . $uid . "\");'>" .
             ((strcmp($action, "editRecord") == 0) ? i("xBbzbd|Change record") : i("wPzFyf|Add record")) . "</a></li>";
    echo "<li><a href='#' onclick='handleEvent(\"deleteRecord_" . $tablename . "_" . $uid . "\");'>" .
             i("2OKHJj|Delete record") . "</a></li>";
    if ($is_versionized_first_period)
        echo "<li><a href='#' onclick='handleEvent(\"editValidfrom_" . $tablename . "_" . $uid . "\");'>" .
                 i("92WEDQ|Edit period start") . "</a></li>";
    if ($is_versionized_last_period) {
        if (floatval($toDisplay["invalid_from"]) < Tfyh_data::forever_seconds)
            echo "<li><a href='#' onclick='handleEvent(\"extendPeriod_" . $tablename . "_" . $uid . "\");'>" .
                     i("bF1lPH|Extend period end") . "</a></li>";
        else {
            echo "<li><a href='#' onclick='handleEvent(\"limitPeriod_" . $tablename . "_" . $uid . "\");'>" .
                     i("mdAwDZ|Limit period end") . "</a></li>";
            echo "<li><a href='#' onclick='handleEvent(\"newPeriod_" . $tablename . "_" . $uid . "\");'>" .
                     i("JX3YTz|Create a new period reco...") . "</a></li>";
        }
    }
    echo "</ol>";
}

echo "</div>";
end_script();
